@@ -2,6 +2,7 @@
 Copyright (c) The Exim Maintainers 2022
 Copyright 1989 by the Massachusetts Institute of Technology,
 Cambridge, Massachusetts.
+SPDX-License-Identifier: GPL-2.0-or-later
 
                         All Rights Reserved
 
@@ -94,7 +95,7 @@ static void  SetResource(Widget, char *, XtArgVal);
 static void  SetSearchLabels(struct SearchAndReplace *, String, String,
   Boolean);
 static Widget CreateDialog(Widget, String, String,
-  void (*)(Widget, char *, Widget));
+		void (*func)(Widget, String, Widget));
 static Widget  GetShell(Widget);
 static void SetWMProtocolTranslations(Widget w);
 static Boolean DoSearch(struct SearchAndReplace *);
@@ -636,10 +637,8 @@ XEvent *event;
  */
 
 static Widget
-CreateDialog(parent, ptr, name, func)
-Widget parent;
-String ptr, name;
-void (*func)();
+CreateDialog(Widget parent, String ptr, String name,
+		void (*func)(Widget form, String ptr, Widget parent))
 {
   Widget popup, form;
   Arg args[5];
