@@ -2,6 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) Jeremy Harris 2019 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -26,7 +27,7 @@ extern auth_external_options_block auth_external_option_defaults;
 
 /* The entry points for the mechanism */
 
-extern void auth_external_init(auth_instance *);
+extern void auth_external_init(driver_instance *);
 extern int auth_external_server(auth_instance *, uschar *);
 extern int auth_external_client(auth_instance *, void *, int, uschar *, int);
 
