@@ -2,6 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 2025 */
 /* Copyright (c) Jeremy Harris 2015 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -25,7 +26,7 @@ extern auth_tls_options_block auth_tls_option_defaults;
 
 /* The entry points for the mechanism */
 
-extern void auth_tls_init(auth_instance *);
+extern void auth_tls_init(driver_instance *);
 extern int auth_tls_server(auth_instance *, uschar *);
 
 /* End of tls.h */
