@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -128,7 +128,7 @@ never_error(uschar *log_msg, uschar *smtp_msg, int was_errno)
 uschar *emsg = was_errno <= 0
   ? US"" : string_sprintf(": %s", strerror(was_errno));
 log_write(0, LOG_MAIN|LOG_PANIC, "%s%s", log_msg, emsg);
-if (smtp_out) smtp_printf("421 %s\r\n", SP_NO_MORE, smtp_msg);
+if (smtp_out_fd >= 0) smtp_printf("421 %s\r\n", SP_NO_MORE, smtp_msg);
 }
 
 
@@ -182,12 +182,11 @@ Returns:            nothing
 
 static void
 handle_smtp_call(struct pollfd * fd_polls, int listen_socket_count,
-  int accept_socket, struct sockaddr *accepted)
+  int accept_socket, const struct sockaddr * accepted)
 {
 pid_t pid;
 union sockaddr_46 interface_sockaddr;
 EXIM_SOCKLEN_T ifsize = sizeof(interface_sockaddr);
-int dup_accept_socket = -1;
 int max_for_this_host = 0;
 int save_log_selector = *log_selector;
 gstring * whofrom;
@@ -205,26 +204,15 @@ DEBUG(D_any) debug_printf("Connection request from %s port %d\n",
 input stream. These operations fail only the exceptional circumstances. Note
 that never_error() won't use smtp_out if it is NULL. */
 
-if (!(smtp_out = fdopen(accept_socket, "wb")))
-  {
-  never_error(US"daemon: fdopen() for smtp_out failed", US"", errno);
-  goto ERROR_RETURN;
-  }
+smtp_out_fd = accept_socket;
 
-if ((dup_accept_socket = dup(accept_socket)) < 0)
+if ((smtp_in_fd = dup(accept_socket)) < 0)
   {
   never_error(US"daemon: couldn't dup socket descriptor",
     US"Connection setup failed", errno);
   goto ERROR_RETURN;
   }
 
-if (!(smtp_in = fdopen(dup_accept_socket, "rb")))
-  {
-  never_error(US"daemon: fdopen() for smtp_in failed",
-    US"Connection setup failed", errno);
-  goto ERROR_RETURN;
-  }
-
 /* Get the data for the local interface address. Panic for most errors, but
 "connection reset by peer" just means the connection went away. */
 
@@ -251,8 +239,10 @@ if (LOGGING(incoming_port))
   whofrom = string_fmt_append(whofrom, ":%d", sender_host_port);
 
 if (LOGGING(incoming_interface))
-  whofrom = string_fmt_append(whofrom, " I=[%s]:%d",
-    interface_address, interface_port);
+  {
+  whofrom = string_fmt_append(whofrom, " I=[%s]", interface_address);
+  whofrom = log_portnum(whofrom, interface_port);
+  }
 
 /* Check maximum number of connections. We do not check for reserved
 connections or unacceptable hosts here. That is done in the subprocess because
@@ -324,7 +314,8 @@ if (smtp_accept_max_per_host)
 tedious per host_address checks. Note that at this stage smtp_accept_count
 contains the count of *other* connections, not including this one. */
 
-if (max_for_this_host > 0 && smtp_accept_count >= max_for_this_host)
+if (  smtp_slots
+   && max_for_this_host > 0 && smtp_accept_count >= max_for_this_host)
   {
   int host_accept_count = 0;
   int other_host_count = 0;    /* keep a count of non matches to optimise */
@@ -382,7 +373,7 @@ if (pid == 0)
 #endif
 
   smtp_accept_count++;    /* So that it includes this process */
-  connection_id = getpid();
+  set_connection_id();
 
   /* Log the connection if requested.
   In order to minimize the cost (because this is going to happen for every
@@ -392,9 +383,9 @@ if (pid == 0)
   arrange to unset the selector in the subprocess.
 
   jgh 2023/08/08 :- moved this logging in from the parent process, just
-  pre-fork.  There was a claim back from 2004 that smtp_accept_count could have
-  become out-of-date by the time the child could log it, and I can't see how
-  that could happen. */
+  pre-fork.  There was a claim back from 4.21 (when it was moved from
+  smtp_start_session()) that smtp_accept_count could have become out-of-date by
+  the time the child could log it, and I can't see how that could happen. */
 
   if (LOGGING(smtp_connection))
     {
@@ -404,7 +395,8 @@ if (pid == 0)
       save_log_selector &= ~L_smtp_connection;
     else if (LOGGING(connection_id))
       log_write(L_smtp_connection, LOG_MAIN, "SMTP connection from %Y "
-	"Ci=%lu (TCP/IP connection count = %d)", whofrom, connection_id, smtp_accept_count);
+	"Ci=%s (TCP/IP connection count = %d)",
+	whofrom, connection_id, smtp_accept_count);
     else
       log_write(L_smtp_connection, LOG_MAIN, "SMTP connection from %Y "
 	"(TCP/IP connection count = %d)", whofrom, smtp_accept_count);
@@ -443,12 +435,11 @@ if (pid == 0)
       {
       if (!f.expand_string_forcedfail)
         {
-        log_write(0, LOG_MAIN|LOG_PANIC, "failed to expand \"%s\" "
+        log_write(0, LOG_MAIN|LOG_PANIC, "failed to expand %q "
           "(smtp_active_hostname): %s", raw_active_hostname,
           expand_string_message);
         smtp_printf("421 Local configuration error; "
           "please try again later.\r\n", SP_NO_MORE);
-        mac_smtp_fflush();
         search_tidyup();
         exim_underbar_exit(EXIT_FAILURE);
         }
@@ -474,8 +465,8 @@ if (pid == 0)
   to be able to communicate with them, under any circumstances. */
   (void)fcntl(accept_socket, F_SETFD,
               fcntl(accept_socket, F_GETFD) | FD_CLOEXEC);
-  (void)fcntl(dup_accept_socket, F_SETFD,
-              fcntl(dup_accept_socket, F_GETFD) | FD_CLOEXEC);
+  (void)fcntl(smtp_in_fd, F_SETFD,
+              fcntl(smtp_in_fd, F_GETFD) | FD_CLOEXEC);
 
 #ifdef SA_NOCLDWAIT
   act.sa_handler = SIG_IGN;
@@ -536,7 +527,7 @@ if (pid == 0)
 
   if (!smtp_start_session())
     {
-    mac_smtp_fflush();
+    smtp_fflush(SFF_NO_UNCORK);
     search_tidyup();
     exim_underbar_exit(EXIT_SUCCESS);
     }
@@ -552,35 +543,24 @@ if (pid == 0)
 
     /* Smtp_setup_msg() returns 0 on QUIT or if the call is from an
     unacceptable host or if an ACL "drop" command was triggered, -1 on
-    connection lost, and +1 on validly reaching DATA. Receive_msg() almost
-    always returns TRUE when smtp_input is true; just retry if no message was
-    accepted (can happen for invalid message parameters). However, it can yield
-    FALSE if the connection was forcibly dropped by the DATA ACL. */
+    connection lost or synprot-error, and +1 on validly reaching DATA.
+    Receive_msg() almost always returns TRUE when smtp_input is true; just retry
+    if no message was accepted (can happen for invalid message parameters).
+    However, it can yield FALSE if the connection was forcibly dropped by the
+    DATA ACL. */
 
-    if ((rc = smtp_setup_msg()) > 0)
+    if ((rc = smtp_setup_msg()) <= 0)		/* bad smtp_setup_msg() */
       {
-      BOOL ok = receive_msg(FALSE);
-      search_tidyup();                    /* Close cached databases */
-      if (!ok)                            /* Connection was dropped */
-        {
-	cancel_cutthrough_connection(TRUE, US"receive dropped");
-        mac_smtp_fflush();
-        smtp_log_no_mail();               /* Log no mail if configured */
-        exim_underbar_exit(EXIT_SUCCESS);
-        }
-      if (!message_id[0]) continue;	/* No message was accepted */
-      }
-    else				/* bad smtp_setup_msg() */
-      {
-      if (smtp_out)
+      if (smtp_out_fd >= 0 && smtp_in_fd >= 0)
 	{
-	int fd = fileno(smtp_in);
 	uschar buf[128];
 
-	mac_smtp_fflush();
+	smtp_fflush(SFF_NO_UNCORK);
+
 	/* drain socket, for clean TCP FINs */
-	if (fcntl(fd, F_SETFL, O_NONBLOCK) == 0)
-	  for(int i = 16; read(fd, buf, sizeof(buf)) > 0 && i > 0; ) i--;
+	if (fcntl(smtp_in_fd, F_SETFL, O_NONBLOCK) == 0)
+	  for(int i = 16; read(smtp_in_fd, buf, sizeof(buf)) > 0 && i > 0; )
+	    i--;
 	}
       cancel_cutthrough_connection(TRUE, US"message setup dropped");
       search_tidyup();
@@ -592,6 +572,19 @@ if (pid == 0)
       exim_underbar_exit(rc ? EXIT_FAILURE : EXIT_SUCCESS);
       }
 
+     {
+      BOOL ok = receive_msg(FALSE);
+      search_tidyup();                    /* Close cached databases */
+      if (!ok)                            /* Connection was dropped */
+        {
+	cancel_cutthrough_connection(TRUE, US"receive dropped");
+        smtp_fflush(SFF_NO_UNCORK);
+        smtp_log_no_mail();               /* Log no mail if configured */
+        exim_underbar_exit(EXIT_SUCCESS);
+        }
+      if (!message_id[0]) continue;	/* No message was accepted */
+     }
+
     /* Show the recipients when debugging */
 
     DEBUG(D_receive)
@@ -697,10 +690,9 @@ if (pid == 0)
 
       if ((dpid = exim_fork(US"daemon-accept-delivery")) == 0)
         {
-        (void)fclose(smtp_in);
-	(void)close(fileno(smtp_out));
-        (void)fclose(smtp_out);
-	smtp_in = smtp_out = NULL;
+        (void)close(smtp_in_fd);
+	(void)close(smtp_out_fd);
+	smtp_in_fd = smtp_out_fd = -1;
 
         /* Don't ever molest the parent's SSL connection, but do clean up
         the data structures if necessary. */
@@ -752,7 +744,7 @@ remember the pid for ticking off when the child completes. */
 
 if (pid < 0)
   never_error(US"daemon: accept process fork failed", US"Fork failed", errno);
-else
+else if (smtp_slots)
   {
   for (int i = 0; i < smtp_accept_max; ++i)
     if (smtp_slots[i].pid <= 0)
@@ -780,23 +772,21 @@ manifest itself as a broken pipe, so drop that one too. If the streams don't
 exist, something went wrong while setting things up. Make sure the socket
 descriptors are closed, in order to drop the connection. */
 
-if (smtp_out)
+if (smtp_out_fd >= 0)
   {
-  if (fclose(smtp_out) != 0 && errno != ECONNRESET && errno != EPIPE)
-    log_write(0, LOG_MAIN|LOG_PANIC, "daemon: fclose(smtp_out) failed: %s",
+  if (close(smtp_out_fd) != 0 && errno != ECONNRESET && errno != EPIPE)
+    log_write(0, LOG_MAIN|LOG_PANIC, "daemon: close(smtp_out_fd) failed: %s",
       strerror(errno));
-  smtp_out = NULL;
+  smtp_out_fd = -1;
   }
-else (void)close(accept_socket);
 
-if (smtp_in)
+if (smtp_in_fd >= 0)
   {
-  if (fclose(smtp_in) != 0 && errno != ECONNRESET && errno != EPIPE)
-    log_write(0, LOG_MAIN|LOG_PANIC, "daemon: fclose(smtp_in) failed: %s",
+  if (close(smtp_in_fd) != 0 && errno != ECONNRESET && errno != EPIPE)
+    log_write(0, LOG_MAIN|LOG_PANIC, "daemon: close(smtp_in_fd) failed: %s",
       strerror(errno));
-  smtp_in = NULL;
+  smtp_in_fd = -1;
   }
-else (void)close(dup_accept_socket);
 
 /* Release any store used in this process, including the store used for holding
 the incoming host address and an expanded active_hostname. */
@@ -896,7 +886,7 @@ while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
   {
   DEBUG(D_any)
     {
-    debug_printf("child %d ended: status=0x%x\n", (int)pid, status);
+    debug_printf("child %ld ended: status=0x%x\n", (long)pid, status);
 #ifdef WCOREDUMP
     if (WIFEXITED(status))
       debug_printf("  normal exit, %d\n", WEXITSTATUS(status));
@@ -912,12 +902,13 @@ while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
   if (smtp_slots)
     {
     int i;
-    for (i = 0; i < smtp_accept_max; i++)
-      if (smtp_slots[i].pid == pid)
+    smtp_slot * sp;
+    for (i = 0, sp = smtp_slots; i < smtp_accept_max; i++, sp++)
+      if (sp->pid == pid)
         {
-        if (smtp_slots[i].host_address)
-          store_free(smtp_slots[i].host_address);
-        smtp_slots[i] = empty_smtp_slot;
+        if (sp->host_address)
+          store_free(sp->host_address);
+        *sp = empty_smtp_slot;
         if (--smtp_accept_count < 0) smtp_accept_count = 0;
         DEBUG(D_any) debug_printf("%d SMTP accept process%s now running\n",
           smtp_accept_count, smtp_accept_count == 1 ? "" : "es");
@@ -967,7 +958,8 @@ if (!*pid_file_path)
   pid_file_path = string_sprintf("%s/exim-daemon.pid", spool_directory);
 
 if (pid_file_path[0] != '/')
-  log_write(0, LOG_PANIC_DIE, "pid file path %s must be absolute\n", pid_file_path);
+  log_write_die(0, LOG_PANIC_DIE,
+		"pid file path %s must be absolute\n", pid_file_path);
 }
 
 
@@ -983,7 +975,7 @@ static BOOL
 operate_on_pid_file(const enum pid_op operation, const pid_t pid)
 {
 char pid_line[sizeof(int) * 3 + 2];
-const int pid_len = snprintf(pid_line, sizeof(pid_line), "%d\n", (int)pid);
+const int pid_len = snprintf(pid_line, sizeof(pid_line), "%ld\n", (long)pid);
 BOOL lines_match = FALSE;
 uschar * path, * base, * dir;
 
@@ -1001,12 +993,12 @@ if (pid_len < 2 || pid_len >= (int)sizeof(pid_line)) goto cleanup;
 
 path = string_copy(pid_file_path);
 if ((base = Ustrrchr(path, '/')) == NULL)	/* should not happen, but who knows */
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid file path \"%s\" does not contain a '/'", pid_file_path);
+  log_write_die(0, LOG_MAIN, "pid file path %q does not contain a '/'", pid_file_path);
 
 dir = base != path ? path : US"/";
 *base++ = '\0';
 
-if (!dir || !*dir || *dir != '/') goto cleanup;
+if (!dir || *dir != '/') goto cleanup;
 if (!base || !*base || Ustrchr(base, '/') != NULL) goto cleanup;
 
 cwd_fd = open(".", dir_flags);
@@ -1018,7 +1010,7 @@ if (dir_fd < 0 || fstat(dir_fd, &sb) != 0 || !S_ISDIR(sb.st_mode)) goto cleanup;
 if (fchdir(dir_fd) != 0) goto cleanup;
 base_fd = open(CS base, O_RDONLY | base_flags);
 if (fchdir(cwd_fd) != 0)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "can't return to previous working dir: %s", strerror(errno));
+  log_write_die(0, LOG_MAIN, "can't return to previous working dir: %s", strerror(errno));
 
 if (base_fd >= 0)
   {
@@ -1049,7 +1041,7 @@ if (operation == PID_WRITE)
       if (fchdir(dir_fd) != 0) goto cleanup;
       error = unlink(CS base);
       if (fchdir(cwd_fd) != 0)
-        log_write(0, LOG_MAIN|LOG_PANIC_DIE, "can't return to previous working dir: %s", strerror(errno));
+        log_write_die(0, LOG_MAIN, "can't return to previous working dir: %s", strerror(errno));
       if (error) goto cleanup;
       (void)close(base_fd);
       base_fd = -1;
@@ -1058,7 +1050,7 @@ if (operation == PID_WRITE)
     if (fchdir(dir_fd) != 0) goto cleanup;
     base_fd = open(CS base, O_WRONLY | O_CREAT | O_EXCL | base_flags, base_mode);
     if (fchdir(cwd_fd) != 0)
-        log_write(0, LOG_MAIN|LOG_PANIC_DIE, "can't return to previous working dir: %s", strerror(errno));
+        log_write_die(0, LOG_MAIN, "can't return to previous working dir: %s", strerror(errno));
     if (base_fd < 0) goto cleanup;
     if (fchmod(base_fd, base_mode) != 0) goto cleanup;
     if (write(base_fd, pid_line, pid_len) != pid_len) goto cleanup;
@@ -1075,7 +1067,7 @@ else
     if (fchdir(dir_fd) != 0) goto cleanup;
     error = unlink(CS base);
     if (fchdir(cwd_fd) != 0)
-        log_write(0, LOG_MAIN|LOG_PANIC_DIE, "can't return to previous working dir: %s", strerror(errno));
+        log_write_die(0, LOG_MAIN, "can't return to previous working dir: %s", strerror(errno));
     if (error) goto cleanup;
     }
   }
@@ -1113,7 +1105,7 @@ since we may require privs for the containing directory */
 static void
 daemon_die(void)
 {
-int pid;
+pid_t pid;
 
 DEBUG(D_any) debug_printf("SIGTERM/SIGINT seen\n");
 #if !defined(DISABLE_TLS) && (defined(EXIM_HAVE_INOTIFY) || defined(EXIM_HAVE_KEVENT))
@@ -1321,8 +1313,8 @@ for (struct cmsghdr * cp = CMSG_FIRSTHDR(&msg);
   struct ucred * cr = (struct ucred *) CMSG_DATA(cp);
   if (cr->uid && cr->uid != exim_uid)
     {
-    DEBUG(D_queue_run) debug_printf("%s: sender creds pid %d uid %d gid %d\n",
-      __FUNCTION__, (int)cr->pid, (int)cr->uid, (int)cr->gid);
+    DEBUG(D_queue_run) debug_printf("%s: sender creds pid %ld uid %d gid %d\n",
+      __FUNCTION__, (long)cr->pid, (int)cr->uid, (int)cr->gid);
     }
 # elif defined(LOCAL_CREDS)				/* BSD-ish */
   struct sockcred * cr = (struct sockcred *) CMSG_DATA(cp);
@@ -1358,13 +1350,13 @@ switch (buf[0])
 
   case NOTIFY_QUEUE_SIZE_REQ:
     {
-    uschar buf[16];
-    int len = snprintf(CS buf, sizeof(buf), "%u", queue_count_cached());
+    uschar qsbuf[16];
+    int len = snprintf(CS qsbuf, sizeof(qsbuf), "%u", queue_count_cached());
 
     DEBUG(D_queue_run)
-      debug_printf("%s: queue size request: %s\n", __FUNCTION__, buf);
+      debug_printf("%s: queue size request: %s\n", __FUNCTION__, qsbuf);
 
-    if (sendto(daemon_notifier_fd, buf, len, 0,
+    if (sendto(daemon_notifier_fd, qsbuf, len, 0,
 		(const struct sockaddr *)&sa_un, msg.msg_namelen) < 0)
       log_write(0, LOG_MAIN|LOG_PANIC,
 	"%s: sendto: %s\n", __FUNCTION__, strerror(errno));
@@ -1754,7 +1746,7 @@ if (f.inetd_wait_mode)
   listen_socket_count = 1;
   (void) close(3);
   if (dup2(0, 3) == -1)
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+    log_write_die(0, LOG_MAIN,
         "failed to dup inetd socket safely away: %s", strerror(errno));
 
   fd_polls[0].fd = 3;
@@ -1783,7 +1775,7 @@ if (f.inetd_wait_mode)
 
   if (tcp_nodelay)
     if (setsockopt(3, IPPROTO_TCP, TCP_NODELAY, US &on, sizeof(on)))
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "failed to set socket NODELAY: %s",
+      log_write_die(0, LOG_MAIN, "failed to set socket NODELAY: %s",
 	strerror(errno));
   }
 
@@ -1872,14 +1864,12 @@ write to stderr. */
 
 if (f.daemon_listen && !f.inetd_wait_mode)
   {
-  int *default_smtp_port;
-  int sep;
+  int * default_smtp_port;
   int pct = 0;
-  uschar *s;
+  uschar * s;
   const uschar * list;
-  uschar *local_iface_source = US"local_interfaces";
-  ip_address_item *ipa;
-  ip_address_item **pipa;
+  uschar * local_iface_source = US"local_interfaces";
+  ip_address_item * ipa, ** pipa;
 
   /* If -oX was used, disable the writing of a pid file unless -oP was
   explicitly used to force it. Then scan the string given to -oX. Any items
@@ -1888,23 +1878,26 @@ if (f.daemon_listen && !f.inetd_wait_mode)
 
   if (override_local_interfaces)
     {
-    gstring * new_smtp_port = NULL;
-    gstring * new_local_interfaces = NULL;
+    gstring * new_smtp_port = NULL, * new_local_interfaces = NULL;
+    uschar joinstr[4] = {0};		/* cppcheck silencing */
 
     if (!override_pid_file_path) write_pid = FALSE;
 
-    list = override_local_interfaces;
-    sep = 0;
-    while ((s = string_nextinlist(&list, &sep, NULL, 0)))
-      {
-      uschar joinstr[4];
-      gstring ** gp = Ustrpbrk(s, ".:") ? &new_local_interfaces : &new_smtp_port;
+    /* specifically permit change-of-separator for admins (who should be
+    the only people getting here, but we may as well be careful) */
 
+    list = f.admin_user
+      ? string_copy_taint(override_local_interfaces, GET_UNTAINTED)
+      : override_local_interfaces;
+
+    for (int sep = 0; s = string_nextinlist(&list, &sep, NULL, 0); )
+      {
+      gstring ** gp = Ustrpbrk(s, ".:") ? &new_local_interfaces : &new_smtp_port;
       if (!*gp)
         {
         joinstr[0] = sep;
         joinstr[1] = ' ';
-        *gp = string_catn(*gp, US"<", 1);
+        *gp = string_catn(NULL, US"<", 1);
         }
 
       *gp = string_catn(*gp, joinstr, 2);
@@ -1932,61 +1925,59 @@ if (f.daemon_listen && !f.inetd_wait_mode)
   build a translated list in a vector. */
 
   list = daemon_smtp_port;
-  sep = 0;
-  while ((s = string_nextinlist(&list, &sep, NULL, 0)))
+  for (int sep = 0; s = string_nextinlist(&list, &sep, NULL, 0); )
     pct++;
   default_smtp_port = store_get((pct+1) * sizeof(int), GET_UNTAINTED);
   list = daemon_smtp_port;
-  sep = 0;
-  for (pct = 0;
-       (s = string_nextinlist(&list, &sep, NULL, 0));
-       pct++)
-    {
+  pct= 0;
+  for (int sep = 0; s = string_nextinlist(&list, &sep, NULL, 0); pct++)
     if (isdigit(*s))
       {
-      uschar *end;
+      uschar * end;
       default_smtp_port[pct] = Ustrtol(s, &end, 0);
-      if (end != s + Ustrlen(s))
-        log_write(0, LOG_PANIC_DIE|LOG_CONFIG, "invalid SMTP port: %s", s);
+      if (*end)
+        log_write_die(0, LOG_CONFIG, "invalid SMTP port: %s", s);
       }
     else
       {
-      struct servent *smtp_service = getservbyname(CS s, "tcp");
+      struct servent * smtp_service = getservbyname(CS s, "tcp");
       if (!smtp_service)
-        log_write(0, LOG_PANIC_DIE|LOG_CONFIG, "TCP port \"%s\" not found", s);
+        log_write_die(0, LOG_CONFIG, "TCP port %q not found", s);
       default_smtp_port[pct] = ntohs(smtp_service->s_port);
       }
-    }
   default_smtp_port[pct] = 0;
 
+#ifndef DISABLE_TLS
   /* Check the list of TLS-on-connect ports and do name lookups if needed */
 
-  list = tls_in.on_connect_ports;
-  sep = 0;
+  list = tls_on_connect_ports;
   /* the list isn't expanded so cannot be tainted.  If it ever is we will trap here */
-  while ((s = string_nextinlist(&list, &sep, big_buffer, big_buffer_size)))
+  for (int sep = 0;
+      s = string_nextinlist(&list, &sep, big_buffer, big_buffer_size); )
     if (!isdigit(*s))
       {
       gstring * g = NULL;
 
-      list = tls_in.on_connect_ports;
-      tls_in.on_connect_ports = NULL;
+      list = tls_on_connect_ports;
+      tls_on_connect_ports = NULL;
       sep = 0;
       while ((s = string_nextinlist(&list, &sep, big_buffer, big_buffer_size)))
-	{
-        if (!isdigit(*s))
+        if (isdigit(*s))
+	  g = string_append_listele(g, ':', s);
+	else
 	  {
 	  struct servent * smtp_service = getservbyname(CS s, "tcp");
 	  if (!smtp_service)
-	    log_write(0, LOG_PANIC_DIE|LOG_CONFIG, "TCP port \"%s\" not found", s);
-	  s = string_sprintf("%d", (int)ntohs(smtp_service->s_port));
+	    log_write_die(0, LOG_CONFIG, "TCP port %q not found", s);
+	  g = string_append_listele_fmt(g, ':', FALSE, "%d",
+					      (int)ntohs(smtp_service->s_port));
 	  }
-	g = string_append_listele(g, ':', s);
-	}
+
       if (g)
-	tls_in.on_connect_ports = g->s;
+	tls_on_connect_ports = g->s;
       break;
       }
+#endif	/*DISABLE_TLS*/
 
   /* Create the list of local interfaces, possibly with ports included. This
   list may contain references to 0.0.0.0 and ::0 as wildcards. These special
@@ -2015,8 +2006,8 @@ if (f.daemon_listen && !f.inetd_wait_mode)
 
     if (ipa->port > 0) continue;
 
-    if (daemon_smtp_port[0] <= 0)
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "no port specified for interface "
+    if (!*daemon_smtp_port)
+      log_write_die(0, LOG_MAIN, "no port specified for interface "
         "%s and daemon_smtp_port is unset; cannot start daemon",
         ipa->address[0] == 0 ? US"\"all IPv4\"" :
         ipa->address[1] == 0 ? US"\"all IPv6\"" : ipa->address);
@@ -2155,7 +2146,7 @@ if (f.background_daemon)
     {
     BOOL daemon_listen = f.daemon_listen;
     pid_t pid = exim_fork(US"daemon");
-    if (pid < 0) log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+    if (pid < 0) log_write_die(0, LOG_MAIN,
       "fork() failed when starting daemon: %s", strerror(errno));
     if (pid > 0) exim_exit(EXIT_SUCCESS); /* in parent process, just exit */
     (void)setsid();                       /* release controlling terminal */
@@ -2204,7 +2195,7 @@ if (f.daemon_listen && !f.inetd_wait_mode)
           "listening (%s): will use IPv4", strerror(errno));
         goto SKIP_SOCKET;
         }
-      log_write(0, LOG_PANIC_DIE, "IPv%c socket creation failed: %s",
+      log_write_die(0, LOG_PANIC_DIE, "IPv%c socket creation failed: %s",
         af == AF_INET6 ? '6' : '4', strerror(errno));
       }
 
@@ -2224,7 +2215,7 @@ if (f.daemon_listen && !f.inetd_wait_mode)
     smtp port for listening. */
 
     if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0)
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "setting SO_REUSEADDR on socket "
+      log_write_die(0, LOG_MAIN, "setting SO_REUSEADDR on socket "
         "failed when starting daemon: %s", strerror(errno));
 
     /* Set TCP_NODELAY; Exim does its own buffering. There is a switch to
@@ -2263,7 +2254,7 @@ if (f.daemon_listen && !f.inetd_wait_mode)
 	: US"(any IPv4)"
 	: ipa->address;
       if (daemon_startup_retries <= 0)
-        log_write(0, LOG_MAIN|LOG_PANIC_DIE,
+        log_write_die(0, LOG_MAIN,
           "socket bind() to port %d for address %s failed: %s: "
           "daemon abandoned", ipa->port, addr, msg);
       log_write(0, LOG_MAIN, "socket bind() to port %d for address %s "
@@ -2315,7 +2306,7 @@ if (f.daemon_listen && !f.inetd_wait_mode)
     where the IPv6 socket accepts both kinds of call. */
 
     if (!check_special_case(errno, addresses, ipa, TRUE))
-      log_write(0, LOG_PANIC_DIE, "listen() failed on interface %s: %s",
+      log_write_die(0, LOG_PANIC_DIE, "listen() failed on interface %s: %s",
         wildcard
 	? af == AF_INET6 ? US"(any IPv6)" : US"(any IPv4)" : ipa->address,
         strerror(errno));
@@ -2432,7 +2423,7 @@ if (f.inetd_wait_mode)
     sprintf(CS p, "with no wait timeout");
 
   log_write(0, LOG_MAIN,
-    "exim %s daemon started: pid=%d, launched with listening socket, %s",
+    "exim %s daemon started: pid=%ld, launched with listening socket, %s",
     version_string, getpid(), big_buffer);
   set_process_info("daemon(%s): pre-listening socket", version_string);
 
@@ -2538,7 +2529,7 @@ else if (f.daemon_listen)
     }
 
   log_write(0, LOG_MAIN,
-    "exim %s daemon started: pid=%d, %s, listening for %s",
+    "exim %s daemon started: pid=%ld, %s, listening for %s",
     version_string, getpid(), qinfo, big_buffer);
   set_process_info("daemon(%s): %s, listening for %s",
     version_string, qinfo, big_buffer);
@@ -2548,7 +2539,7 @@ else	/* no listening sockets, only queue-runs */
   {
   const uschar * s = describe_queue_runners();
   log_write(0, LOG_MAIN,
-    "exim %s daemon started: pid=%d, %s, not listening for SMTP",
+    "exim %s daemon started: pid=%ld, %s, not listening for SMTP",
     version_string, getpid(), s);
   set_process_info("daemon(%s): %s, not listening", version_string, s);
   }
@@ -2559,25 +2550,9 @@ else	/* no listening sockets, only queue-runs */
 dns_pattern_init();
 smtp_deliver_init();	/* Used for callouts */
 
-#ifndef DISABLE_DKIM
-  {
-# ifdef MEASURE_TIMING
-  struct timeval t0;
-  gettimeofday(&t0, NULL);
-# endif
-  dkim_exim_init();
-# ifdef MEASURE_TIMING
-  report_time_since(&t0, US"dkim_exim_init (delta)");
-# endif
-  }
-#endif
-
 #ifdef WITH_CONTENT_SCAN
 malware_init();
 #endif
-#ifdef SUPPORT_SPF
-spf_init();
-#endif
 #ifndef DISABLE_TLS
 tls_daemon_init();
 #endif
@@ -2869,7 +2844,7 @@ for (;;)
 
   if (sighup_seen)
     {
-    log_write(0, LOG_MAIN, "pid %d: SIGHUP received: re-exec daemon",
+    log_write(0, LOG_MAIN, "pid %ld: SIGHUP received: re-exec daemon",
       getpid());
     close_daemon_sockets(daemon_notifier_fd, fd_polls, listen_socket_count);
     unlink_notifier_socket();
@@ -2878,9 +2853,9 @@ for (;;)
     sighup_argv[0] = exim_path;
     exim_nullstd();
     execv(CS exim_path, (char *const *)sighup_argv);
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "pid %d: exec of %s failed: %s",
+    log_write_die(0, LOG_MAIN, "pid %ld: exec of %s failed: %s",
       getpid(), exim_path, strerror(errno));
-    log_close_all();
+    /*NOTREACHED*/
     }
 
   }   /* End of main loop */
