@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2015 - 2023 */
+/* Copyright (c) The Exim Maintainers 2015 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -46,6 +46,13 @@ const uschar * dns_rc_names[] = {
   [DNS_FAIL] =		US"DNS_FAIL",
 };
 
+const uschar * dsn_aware_names[] = {
+  [dsn_support_unknown] = US"UNKNOWN",
+  [dsn_support_yes] =	  US"YES",
+  [dsn_support_no] =	  US"NO",
+};
+
+
 
 /*************************************************
 *               Print tree                       *
@@ -133,9 +140,9 @@ debug_print_string(uschar *debug_string)
 if (!debug_string) return;
 HDEBUG(D_any|D_v)
   {
-  uschar *s = expand_string(debug_string);
+  uschar * s = expand_string(debug_string);
   if (!s)
-    debug_printf("failed to expand debug_output \"%s\": %s\n", debug_string,
+    debug_printf("failed to expand debug_output %q: %s\n", debug_string,
       expand_string_message);
   else if (s[0] != 0)
     debug_printf("%s%s", s, (s[Ustrlen(s)-1] == '\n')? "" : "\n");
@@ -227,7 +234,7 @@ if (debug_ptr == debug_buffer)
     {
     struct timeval now;
     time_t tmp;
-    struct tm * t;
+    const struct tm * t;
 
     gettimeofday(&now, NULL);
     tmp = now.tv_sec;
@@ -243,10 +250,7 @@ if (debug_ptr == debug_buffer)
   /* Set up prefix if outputting for host checking and not debugging */
 
   if (host_checking && debug_selector == 0)
-    {
-    Ustrcpy(debug_ptr, US">>> ");
-    debug_ptr += 4;
-    }
+    debug_ptr = Ustpcpy(debug_ptr, US">>> ");
 
   debug_prefix_length = debug_ptr - debug_buffer;
   }
@@ -256,19 +260,16 @@ if (indent > 0)
   for (int i = indent >> 2; i > 0; i--)
     DEBUG(D_noutf8)
       {
-      Ustrcpy(debug_ptr, US"   !");
-      debug_ptr += 4;	/* 3 spaces + shriek */
+      debug_ptr = Ustpcpy(debug_ptr, US"   !");
       debug_prefix_length += 4;
       }
     else
       {
-      Ustrcpy(debug_ptr, US"   " UTF8_VERT_2DASH);
-      debug_ptr += 6;	/* 3 spaces + 3 UTF-8 octets */
+      debug_ptr = Ustpcpy(debug_ptr, US"   " UTF8_VERT_2DASH);
       debug_prefix_length += 6;
       }
 
-  Ustrncpy(debug_ptr, US"   ", indent &= 3);
-  debug_ptr += indent;
+  debug_ptr += sprintf(CS debug_ptr, "%.*s", indent &= 3, "   ");
   debug_prefix_length += indent;
   }
 
@@ -284,7 +285,7 @@ we trust that we will never expand the results. */
 		.s = debug_buffer };
   if (!string_vformat(&gs, SVFMT_TAINT_NOCHK, format, ap))
     {
-    uschar * s = US"**** debug string too long - truncated ****\n";
+    const uschar * s = US"**** debug string too long - truncated ****\n";
     uschar * p = gs.s + gs.ptr;
     int maxlen = gs.size - Ustrlen(s) - 2;
     if (p > gs.s + maxlen) p = gs.s + maxlen;
