@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2010 - 2024 */
+/* Copyright (c) The Exim Maintainers 2010 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -88,8 +88,8 @@ while (c && *p)
 return TRUE;
 
 bad:
-  if (panic) log_write(0, LOG_MAIN|LOG_PANIC_DIE,
-    "Failed to %s directory \"%s\": %s\n", p, path, exim_errstr(errno));
+  if (panic) log_write_die(0, LOG_MAIN,
+    "Failed to %s directory %q: %s\n", p, path, exim_errstr(errno));
   return FALSE;
 }
 
