@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -19,32 +19,28 @@ are in in fact in separate headers. */
 #include <sys/time.h>
 
 
-#ifdef EXIM_PERL
-extern gstring *call_perl_cat(gstring *, uschar **, uschar *,
-                 uschar **) WARN_UNUSED_RESULT;
-extern void    cleanup_perl(void);
-extern uschar *init_perl(uschar *);
-#endif
-
-
 #ifndef DISABLE_TLS
-extern const char *
-               std_dh_prime_default(void);
-extern const char *
-               std_dh_prime_named(const uschar *);
+extern gstring *	add_tls_info_for_log(gstring *);
+extern const char *	std_dh_prime_default(void);
+extern const char *	std_dh_prime_named(const uschar *);
 
-extern uschar * tls_cert_crl_uri(void *, uschar * mod);
+# ifdef TCP_FASTOPEN
+extern void	tfo_out_check(int);
+# endif
+
+# if !defined(COMPILE_UTILITY) && !defined(MACRO_PREDEF)
+extern uschar * tls_cert_crl_uri(void *, const uschar *);
 extern uschar * tls_cert_ext_by_oid(void *, uschar *, int);
-extern uschar * tls_cert_issuer(void *, uschar * mod);
-extern uschar * tls_cert_not_before(void *, uschar * mod);
-extern uschar * tls_cert_not_after(void *, uschar * mod);
-extern uschar * tls_cert_ocsp_uri(void *, uschar * mod);
-extern uschar * tls_cert_serial_number(void *, uschar * mod);
-extern uschar * tls_cert_signature(void *, uschar * mod);
-extern uschar * tls_cert_signature_algorithm(void *, uschar * mod);
-extern uschar * tls_cert_subject(void *, uschar * mod);
-extern uschar * tls_cert_subject_altname(void *, uschar * mod);
-extern uschar * tls_cert_version(void *, uschar * mod);
+extern uschar * tls_cert_issuer(void *, const uschar *);
+extern uschar * tls_cert_not_before(void *, const uschar *);
+extern uschar * tls_cert_not_after(void *, const uschar *);
+extern uschar * tls_cert_ocsp_uri(void *, const uschar *);
+extern uschar * tls_cert_serial_number(void *, const uschar *);
+extern uschar * tls_cert_signature(void *, const uschar *);
+extern uschar * tls_cert_signature_algorithm(void *, const uschar *);
+extern uschar * tls_cert_subject(void *, const uschar *);
+extern uschar * tls_cert_subject_altname(void *, const uschar *);
+extern uschar * tls_cert_version(void *, const uschar *);
 
 extern uschar * tls_cert_der_b64(void * cert);
 extern uschar * tls_cert_fprt_md5(void *);
@@ -52,7 +48,7 @@ extern uschar * tls_cert_fprt_sha1(void *);
 extern uschar * tls_cert_fprt_sha256(void *);
 
 extern void    tls_clean_env(void);
-extern BOOL    tls_client_start(client_conn_ctx *, smtp_connect_args *,
+extern BOOL    tls_client_start(client_conn_ctx *, const smtp_connect_args *,
 		  void *, tls_support *, uschar **);
 extern BOOL    tls_client_adjunct_start(host_item *, client_conn_ctx *,
 		  const uschar *, uschar **);
@@ -73,22 +69,25 @@ extern uschar *tls_getbuf(unsigned *);
 extern void    tls_get_cache(unsigned);
 extern BOOL    tls_hasc(void);
 extern BOOL    tls_import_cert(const uschar *, void **);
+extern void    tls_state_in_to_out(int, const uschar *, int);
+extern void    tls_state_out_to_in(int, const uschar *, int);
 extern BOOL    tls_is_name_for_cert(const uschar *, void *);
-# ifdef USE_OPENSSL
-extern BOOL    tls_openssl_options_parse(uschar *, long *);
-# endif
+#  ifdef USE_OPENSSL
+extern BOOL    tls_openssl_options_parse(const uschar *, long *);
+#  endif
 extern int     tls_read(void *, uschar *, size_t);
-extern int     tls_server_start(uschar **);
+extern int     tls_server_start(uschar **, gstring *);
 extern void    tls_shutdown_wr(void *);
 extern BOOL    tls_smtp_buffered(void);
 extern int     tls_ungetc(int);
-#if defined(EXIM_HAVE_INOTIFY) || defined(EXIM_HAVE_KEVENT)
+#  if defined(EXIM_HAVE_INOTIFY) || defined(EXIM_HAVE_KEVENT)
 extern void    tls_watch_discard_event(int);
 extern void    tls_watch_invalidate(void);
-#endif
+#  endif
 extern int     tls_write(void *, const uschar *, size_t, BOOL);
 extern uschar *tls_validate_require_cipher(void);
 extern gstring *tls_version_report(gstring *);
+# endif	/* !COMPILE_UTILITY && !MACRO_PREDEF */
 
 # ifdef SUPPORT_DANE
 extern int     tlsa_lookup(const host_item *, dns_answer *, BOOL);
@@ -105,25 +104,15 @@ extern uschar *acl_current_verb(void);
 extern int     acl_eval(int, uschar *, uschar **, uschar **);
 extern uschar *acl_standalone_setvar(const uschar *, BOOL);
 
-extern tree_node *acl_var_create(uschar *);
+extern tree_node *acl_var_create(const uschar *);
 extern void    acl_var_write(uschar *, uschar *, void *);
-
-#ifdef EXPERIMENTAL_ARC
-# ifdef SUPPORT_DMARC
-extern gstring *arc_dmarc_hist_append(gstring *);
-# endif
-extern void   *arc_ams_setup_sign_bodyhash(void);
-extern const uschar *arc_header_feed(gstring *, BOOL);
-extern gstring *arc_sign(const uschar *, gstring *, uschar **);
-extern void     arc_sign_init(void);
-extern const uschar *acl_verify_arc(void);
-extern uschar * fn_arc_domains(void);
-#endif
+extern void    add_driver_info(driver_info **, const driver_info *, size_t);
 
 extern void    assert_no_variables(void *, int, const char *, int);
-extern int     auth_call_pam(const uschar *, uschar **);
+extern void    atrn_handle_customer(void);
+extern int     atrn_handle_provider(uschar **, uschar **);
+
 extern int     auth_call_pwcheck(uschar *, uschar **);
-extern int     auth_call_radius(const uschar *, uschar **);
 extern int     auth_call_saslauthd(const uschar *, const uschar *,
 	         const uschar *, const uschar *, uschar **);
 extern int     auth_check_serv_cond(auth_instance *);
@@ -131,29 +120,14 @@ extern int     auth_check_some_cond(auth_instance *, uschar *, uschar *, int);
 extern int     auth_client_item(void *, auth_instance *, const uschar **,
 		 unsigned, int, uschar *, int);
 
-
 extern int     auth_get_data(uschar **, const uschar *, int);
 extern int     auth_get_no64_data(uschar **, uschar *);
 extern int     auth_prompt(const uschar *);
 extern int     auth_read_input(const uschar *);
 extern gstring * auth_show_supported(gstring *);
-extern uschar *auth_xtextencode(uschar *, int);
-extern int     auth_xtextdecode(uschar *, uschar **);
 extern uschar *authenticator_current_name(void);
 
-#ifdef EXPERIMENTAL_ARC
-extern gstring *authres_arc(gstring *);
-#endif
-#ifndef DISABLE_DKIM
-extern gstring *authres_dkim(gstring *);
-#endif
-#ifdef SUPPORT_DMARC
-extern gstring *authres_dmarc(gstring *);
-#endif
 extern gstring *authres_smtpauth(gstring *);
-#ifdef SUPPORT_SPF
-extern gstring *authres_spf(gstring *);
-#endif
 
 extern uschar *b64encode(const uschar *, int);
 extern uschar *b64encode_taint(const uschar *, int, const void *);
@@ -168,8 +142,8 @@ extern void    bits_clear(unsigned int *, size_t, int *);
 extern void    bits_set(unsigned int *, size_t, int *);
 
 extern void    cancel_cutthrough_connection(BOOL, const uschar *);
-extern gstring *cat_file(FILE *, gstring *, uschar *);
-extern gstring *cat_file_tls(void *, gstring *, uschar *);
+extern gstring *cat_file(FILE *, gstring *, const uschar *);
+extern gstring *cat_file_tls(void *, gstring *, const uschar *);
 extern int     check_host(void *, const uschar *, const uschar **, uschar **);
 extern uschar **child_exec_exim(int, BOOL, int *, BOOL, int, ...);
 extern pid_t   child_open_exim_function(int *, const uschar *);
@@ -228,14 +202,7 @@ extern void    delivery_re_exec(int);
 
 extern void    die_tainted(const uschar *, const uschar *, int);
 extern BOOL    directory_make(const uschar *, const uschar *, int, BOOL);
-#ifndef DISABLE_DKIM
-extern uschar *dkim_exim_query_dns_txt(const uschar *);
-extern void    dkim_exim_sign_init(void);
-
-extern BOOL    dkim_transport_write_message(transport_ctx *,
-		  struct ob_dkim *, const uschar ** errstr);
-#endif
-extern dns_address *dns_address_from_rr(dns_answer *, dns_record *);
+extern dns_address *dns_address_from_rr(const dns_answer *, dns_record *);
 extern int     dns_basic_lookup(dns_answer *, const uschar *, int);
 extern uschar *dns_build_reverse(const uschar *);
 extern time_t  dns_expire_from_soa(dns_answer *, int);
@@ -265,14 +232,26 @@ extern void    exim_nullstd(void);
 extern void    exim_setugid(uid_t, gid_t, BOOL, const uschar *);
 extern void    exim_underbar_exit(int) NORETURN;
 extern void    exim_wait_tick(struct timeval *, int);
-extern int     exp_bool(address_item *addr,
-  uschar *mtype, uschar *mname, unsigned dgb_opt, uschar *oname, BOOL bvalue,
-  uschar *svalue, BOOL *rvalue);
-extern BOOL    expand_check_condition(uschar *, uschar *, uschar *);
+extern int     exp_bool(address_item *,
+  const uschar *, const uschar *, unsigned, uschar *, BOOL bvalue,
+  const uschar *, BOOL *);
+extern BOOL    expand_check_condition(const uschar *, const uschar *, const uschar *);
 extern uschar *expand_file_big_buffer(const uschar *);
-extern uschar *expand_string(uschar *);	/* public, cannot make const */
+
 extern const uschar *expand_string_2(const uschar *, BOOL *);
-extern const uschar *expand_cstring(const uschar *); /* ... so use this one */
+
+static inline uschar * expand_nc_string(uschar * s)
+{ return US expand_string_2(s, NULL); }
+static inline const uschar * expand_c_string(const uschar * s)
+{ return expand_string_2(s, NULL); }
+
+/* A macro that picks which function to use depending on the type of the arg */
+#define expand_string(X) _Generic((X),     \
+	      uschar *:		expand_nc_string, \
+	      const uschar *:	expand_c_string   \
+	      )(X)
+
+extern BOOL   expand_string_nonempty(const uschar *);
 extern uschar *expand_getkeyed(const uschar *, const uschar *);
 
 extern uschar *expand_hide_passwords(uschar * );
@@ -281,11 +260,7 @@ extern int_eximarith_t expand_string_integer(uschar *, BOOL);
 extern void    modify_variable(uschar *, void *);
 
 extern BOOL    fd_ready(int, time_t);
-
-extern int     filter_interpret(const uschar *, int, address_item **, uschar **);
-extern BOOL    filter_personal(string_item *, BOOL);
 extern BOOL    filter_runtest(int, const uschar *, BOOL, BOOL);
-extern BOOL    filter_system_interpret(address_item **, uschar **);
 
 extern uschar * fn_hdrs_added(void);
 extern void    force_fd(int, int);
@@ -303,8 +278,9 @@ extern void    host_build_log_info(void);
 extern void    host_build_sender_fullhost(void);
 extern int     host_find_byname(host_item *, const uschar *, int,
 				const uschar **, BOOL);
-extern int     host_find_bydns(host_item *, const uschar *, int, uschar *, uschar *,
-                 uschar *, const dnssec_domains *, const uschar **, BOOL *);
+extern int     host_find_bydns(host_item *, const uschar *, int, const uschar *,
+		const uschar *, const uschar *, const dnssec_domains *,
+		const uschar **, BOOL *);
 extern ip_address_item *host_find_interfaces(void);
 extern BOOL    host_is_in_net(const uschar *, const uschar *, int);
 extern BOOL    host_is_tls_on_connect_port(int);
@@ -320,7 +296,7 @@ extern uschar *imap_utf7_encode(uschar *, const uschar *,
 
 extern void    invert_address(uschar *, uschar *);
 extern int     ip_addr(void *, int, const uschar *, int);
-extern int     ip_bind(int, int, uschar *, int);
+extern int     ip_bind(int, int, const uschar *, int);
 extern int     ip_connect(int, int, const uschar *, int, int, const blob *);
 extern int     ip_connectedsocket(int, const uschar *, int, int,
                  int, host_item *, uschar **, const blob *);
@@ -336,12 +312,21 @@ extern int     ip_streamsocket(const uschar *, uschar **, int, host_item *);
 extern int     ipv6_nmtoa(int *, uschar *);
 
 extern const uschar *local_part_quote(const uschar *);
-extern int     log_open_as_exim(const uschar * const);
 extern void    log_close_all(void);
+extern int     log_open_as_exim(const uschar * const);
+extern gstring *log_portnum(gstring *, int);
+extern void    log_write_die(unsigned, int, const char * format, ...)
+		PRINTF_FUNCTION(3,4) NORETURN;
+
+extern const lookup_info * lookup_with_acq_num(unsigned);
+#ifdef LOOKUP_MODULE_DIR
+extern BOOL    lookup_one_mod_load(const uschar *, uschar **);
+#endif
+
 
 extern macro_item * macro_create(const uschar *, const uschar *, BOOL);
 extern BOOL    macro_read_assignment(uschar *);
-extern uschar *macros_expand(int, int *, BOOL *);
+extern uschar *macros_expand(int, int *, int *);
 extern void    mainlog_close(void);
 #ifdef WITH_CONTENT_SCAN
 extern int     malware(const uschar *, BOOL, int);
@@ -352,13 +337,15 @@ extern gstring * malware_show_supported(gstring *);
 extern int     match_address_list(const uschar *, BOOL, BOOL, const uschar **,
                  unsigned int *, int, int, const uschar **);
 extern int     match_address_list_basic(const uschar *, const uschar **, int);
-extern int     match_check_list(const uschar **, int, tree_node **, unsigned int **,
-                 int(*)(void *, const uschar *, const uschar **, uschar **), void *, int,
-                 const uschar *, const uschar **);
-extern int     match_isinlist(const uschar *, const uschar **, int, tree_node **,
-                 unsigned int *, int, BOOL, const uschar **);
+extern int     match_check_list(const uschar * const *, int, tree_node **,
+		 unsigned int **, int(*)(void *, const uschar *,
+					 const uschar **, uschar **),
+		 void *, int, const uschar *, const uschar **);
+extern int     match_isinlist(const uschar *, const uschar * const *, int,
+		 tree_node **, unsigned int *, int, BOOL, const uschar **);
 extern int     match_check_string(const uschar *, const uschar *, int, mcs_flags,
                  const uschar **);
+extern uschar  matchlist_parse_sep(const uschar **);
 
 extern void    message_start(void);
 extern void    message_tidyup(void);
@@ -368,27 +355,40 @@ extern void    md5_start(md5 *);
 extern void    millisleep(int);
 #ifdef WITH_CONTENT_SCAN
 struct mime_boundary_context;
-extern int     mime_acl_check(uschar *acl, FILE *f,
+extern int     mime_acl_check(uschar *, FILE *,
                  struct mime_boundary_context *, uschar **, uschar **);
 extern int     mime_decode(const uschar **);
-extern ssize_t mime_decode_base64(FILE *, FILE *, uschar *);
+extern ssize_t mime_decode_base64(FILE *, FILE *, const uschar *);
 extern int     mime_regex(const uschar **, BOOL);
 extern void    mime_set_anomaly(int);
 #endif
+
+extern gstring *misc_mod_authres(gstring *);
+extern int     misc_mod_conn_init(const uschar *, const uschar *,
+		const uschar **);
+extern misc_module_info * misc_mod_find(const uschar * modname, uschar **);
+extern misc_module_info * misc_mod_findonly(const uschar * modname);
+extern int     misc_mod_msg_init(void);
+extern void    misc_mod_smtp_reset(void);
+
 extern uschar *moan_check_errorcopy(const uschar *);
 extern BOOL    moan_skipped_syntax_errors(uschar *, error_block *, uschar *,
                  BOOL, uschar *);
-extern void    moan_smtp_batch(uschar *, const char *, ...) PRINTF_FUNCTION(2,3);
+extern void    moan_smtp_batch(const uschar *, const char *, ...)
+		  PRINTF_FUNCTION(2,3);
 extern BOOL    moan_send_message(const uschar *, int, error_block *eblock,
-		 header_line *, FILE *, uschar *);
-extern void    moan_tell_someone(uschar *, address_item *,
+		 header_line *, FILE *, const uschar *);
+extern void    moan_tell_someone(const uschar *, address_item *,
                  const uschar *, const char *, ...) PRINTF_FUNCTION(4,5);
 extern BOOL    moan_to_sender(int, error_block *, header_line *, FILE *, BOOL);
 extern void    moan_write_from(FILE *);
 extern void    moan_write_references(FILE *, uschar *);
+#ifdef LOOKUP_MODULE_DIR
+//extern void    mod_load_check(const uschar *);
+#endif
 extern FILE   *modefopen(const uschar *, const char *, mode_t);
 
-extern int     open_cutthrough_connection( address_item * addr );
+extern int     open_cutthrough_connection(address_item *, BOOL);
 
 extern uschar *parse_extract_address(const uschar *, uschar **, int *, int *, int *,
                  BOOL);
@@ -401,6 +401,10 @@ extern const uschar *parse_message_id(const uschar *, uschar **, uschar **);
 extern const uschar *parse_quote_2047(const uschar *, int, const uschar *,
 				      BOOL);
 extern const uschar *parse_date_time(const uschar *str, time_t *t);
+#ifdef EXIM_PERL
+const misc_module_info * perl_startup(const uschar *);
+#endif
+
 extern void priv_drop_temp(const uid_t, const gid_t);
 extern void priv_restore(void);
 #ifdef SUPPORT_PROXY
@@ -420,14 +424,14 @@ extern void    queue_run(qrunner *, const uschar *, const uschar *, BOOL);
 
 extern int     random_number(int);
 extern const uschar *rc_to_string(int);
-extern int     rda_interpret(redirect_block *, int, const uschar *, const uschar *,
-                 const uschar *, const uschar *, const uschar *, const ugid_block *, address_item **,
-                 uschar **, error_block **, int *, const uschar *);
+extern int     rda_interpret(redirect_block *, int, const uschar *,
+		const sieve_block *, const ugid_block *, address_item **,
+		uschar **, error_block **, int *, const uschar *);
 extern int     rda_is_filter(const uschar *);
 extern BOOL    readconf_depends(driver_instance *, uschar *);
-extern void    readconf_driver_init(uschar *, driver_instance **,
-                 driver_info *, int, void *, int, optionlist *, int);
-extern const uschar *readconf_find_option(void *);
+extern void    readconf_driver_init(driver_instance **, driver_info **, int,
+		void *, int, optionlist *, int, const uschar *);
+extern const uschar *readconf_find_option(const void *);
 extern void    readconf_main(BOOL);
 extern void    readconf_options_from_list(optionlist *, unsigned, const uschar *, uschar *);
 extern BOOL    readconf_print(const uschar *, const uschar *, BOOL);
@@ -438,12 +442,13 @@ extern void    readconf_rest(void);
 extern uschar *readconf_retry_error(const uschar *, const uschar *, int *, int *);
 extern void    readconf_save_config(const uschar *);
 extern void    read_message_body(BOOL);
-extern void    receive_bomb_out(uschar *, uschar *) NORETURN;
+extern void    receive_bomb_out(const uschar *, uschar *) NORETURN;
 extern BOOL    receive_check_fs(int);
 extern BOOL    receive_check_set_sender(const uschar *);
 extern BOOL    receive_msg(BOOL);
 extern int_eximarith_t receive_statvfs(BOOL, int *);
 extern void    receive_swallow_smtp(void);
+extern int     recv_fd_from_sock(int);
 #ifdef WITH_CONTENT_SCAN
 extern int     regex(const uschar **, BOOL);
 extern void    regex_vars_clear(void);
@@ -452,12 +457,12 @@ extern void    regex_at_daemon(const uschar *);
 extern BOOL    regex_match(const pcre2_code *, const uschar *, int, uschar **);
 extern BOOL    regex_match_and_setup(const pcre2_code *, const uschar *, int, int);
 extern const pcre2_code *regex_compile(const uschar *, mcs_flags, uschar **,
-		pcre2_compile_context *);
+		  pcre2_compile_context *);
 extern const pcre2_code *regex_must_compile(const uschar *, mcs_flags, BOOL);
 
 extern void    retry_add_item(address_item *, const uschar *, int);
-extern BOOL    retry_check_address(const uschar *, host_item *, uschar *, BOOL,
-                 const uschar **, const uschar **);
+extern BOOL    retry_check_address(const uschar *, host_item *, const uschar *,
+		  BOOL, const uschar **, const uschar **);
 extern retry_config *retry_find_config(const uschar *, const uschar *, int, int);
 extern const uschar *retry_host_key_build(const host_item *, BOOL,
 		  const uschar *);
@@ -489,22 +494,24 @@ extern gstring * route_show_supported(gstring *);
 extern void    route_tidyup(void);
 extern uschar *router_current_name(void);
 
-extern uschar *search_args(int, uschar *, uschar *, uschar **, const uschar *);
-extern uschar *search_find(void *, const uschar *, uschar *, int,
+extern uschar *search_args(const lookup_info *, uschar *, uschar *, uschar **,
+		const uschar *);
+extern uschar *search_find(void *, const uschar *, const uschar *, int,
 		 const uschar *, int, int, int *, const uschar *);
-extern int     search_findtype(const uschar *, int);
-extern int     search_findtype_partial(const uschar *, int *, const uschar **, int *,
-                 int *, const uschar **);
-extern void   *search_open(const uschar *, int, int, uid_t *, gid_t *);
+extern const lookup_info * search_findtype(const uschar *, int);
+extern const lookup_info * search_findtype_partial(const uschar *, int *,
+		const uschar **, int *, int *, const uschar **);
+extern void   *search_open(const uschar *, const lookup_info *, int,
+		uid_t *, gid_t *);
 extern void    search_tidyup(void);
+extern BOOL    send_fd_over_socket(int, int);
 extern uschar *sender_helo_verified_boolstr(void);
+#if !defined(COMPILE_UTILITY) && !defined(MACRO_PREDEF)
 extern void    set_process_info(const char *, ...) PRINTF_FUNCTION(1,2);
 extern void    sha1_end(hctx *, const uschar *, int, uschar *);
 extern void    sha1_mid(hctx *, const uschar *);
 extern void    sha1_start(hctx *);
-extern int     sieve_interpret(const uschar *, int, const uschar *,
-		 const uschar *, const uschar *, const uschar *,
-		 address_item **, uschar **);
+#endif
 extern void    sigalrm_handler(int);
 extern void    single_queue_run(qrunner *, const uschar *, const uschar *);
 extern int     smtp_boundsock(smtp_connect_args *);
@@ -519,9 +526,9 @@ extern int     smtp_connect(smtp_connect_args *, const blob *);
 extern int     smtp_feof(void);
 extern int     smtp_ferror(void);
 extern uschar *smtp_get_connection_info(void);
-extern BOOL    smtp_get_interface(uschar *, int, address_item *,
-                 uschar **, uschar *);
-extern BOOL    smtp_get_port(uschar *, address_item *, int *, uschar *);
+extern BOOL    smtp_get_interface(const uschar *, int, address_item *,
+                 const uschar **, const uschar *);
+extern int     smtp_get_port(const uschar *, address_item *, const uschar *);
 extern int     smtp_getc(unsigned);
 extern uschar *smtp_getbuf(unsigned *);
 extern void    smtp_get_cache(unsigned);
@@ -529,24 +536,27 @@ extern BOOL    smtp_hasc(void);
 extern int     smtp_handle_acl_fail(int, int, uschar *, uschar *);
 extern void    smtp_log_no_mail(void);
 extern void    smtp_message_code(uschar **, int *, uschar **, uschar **, BOOL);
+extern void    smtp_notquit_exit(const uschar *, uschar *, const uschar *, ...);
+extern void    smtp_port_for_connect(host_item *, int);
 extern void    smtp_proxy_tls(void *, uschar *, size_t, int *, int, const uschar *) NORETURN;
 extern BOOL    smtp_read_response(void *, uschar *, int, int, int);
-extern void   *smtp_reset(void *);
+rmark	       smtp_reset(rmark);
 extern void    smtp_respond(uschar *, int, BOOL, uschar *);
-extern void    smtp_notquit_exit(uschar *, uschar *, uschar *, ...);
-extern void    smtp_port_for_connect(host_item *, int);
 extern void    smtp_send_prohibition_message(int, uschar *);
 extern int     smtp_setup_msg(void);
 extern int     smtp_sock_connect(smtp_connect_args *, int, const blob *);
 extern BOOL    smtp_start_session(void);
 extern int     smtp_ungetc(int);
+extern void    smtp_verify_feed(const uschar *, unsigned);
 extern BOOL    smtp_verify_helo(void);
+extern int     smtp_write_atrn(address_item *, cut_t *);
 extern int     smtp_write_command(void *, int, const char *, ...) PRINTF_FUNCTION(3,4);
 #ifdef WITH_CONTENT_SCAN
 extern int     spam(const uschar **);
-extern FILE   *spool_mbox(unsigned long *, const uschar *, uschar **);
+extern FILE   *spool_mbox(unsigned long *, const uschar *, const uschar **);
 #endif
 extern void    spool_clear_header_globals(void);
+extern int     spool_has_one_undelivered_dom(const uschar *);
 extern BOOL    spool_move_message(const uschar *, const uschar *, const uschar *, const uschar *);
 extern int     spool_open_datafile(const uschar *);
 extern int     spool_open_temp(uschar *);
@@ -564,9 +574,14 @@ extern void    store_exit(void);
 extern void    store_init(void);
 extern void    store_writeprotect(int);
 
+#ifdef MISSING_POSIX_STPCPY
+extern char * stpcpy(char * restrict, const char * restrict);
+#endif
+
 extern gstring *string_append(gstring *, int, ...) WARN_UNUSED_RESULT;
 extern gstring *string_append_listele(gstring *, uschar, const uschar *) WARN_UNUSED_RESULT;
 extern gstring *string_append_listele_n(gstring *, uschar, const uschar *, unsigned) WARN_UNUSED_RESULT;
+extern gstring *string_append_listele_fmt(gstring *, uschar, BOOL, const char *, ...) WARN_UNUSED_RESULT;
 extern gstring *string_append2_listele_n(gstring *, const uschar *, const uschar *, unsigned) WARN_UNUSED_RESULT;
 extern uschar *string_base62_32(unsigned long int);
 extern uschar *string_base62_64(unsigned long int);
@@ -616,32 +631,31 @@ extern uschar *string_nextinlist_trc(const uschar **listptr, int *separator, usc
 
 extern int     strcmpic(const uschar *, const uschar *);
 extern int     strncmpic(const uschar *, const uschar *, int);
-extern uschar *strstric(uschar *, uschar *, BOOL);
+extern uschar *strstric(const uschar *, const uschar *, BOOL);
 extern const uschar *strstric_c(const uschar *, const uschar *, BOOL);
 
+extern int     synprot_error(int, int, uschar *, uschar *);
+
 extern int     test_harness_fudged_queue_time(int);
 extern void    tcp_init(void);
 #ifdef EXIM_TFO_PROBE
 extern void    tfo_probe(void);
 #endif
+#if !defined(COMPILE_UTILITY) && !defined(MACRO_PREDEF)
 extern void    tls_modify_variables(tls_support *);
+#endif
 extern uschar *tod_stamp(int);
 
 extern BOOL    transport_check_waiting(const uschar *, const uschar *, int, uschar *,
                  oicf, void*);
 extern uschar *transport_current_name(void);
 extern void    transport_do_pass_socket(const uschar *, const uschar *,
-		 const uschar *, uschar *, int);
+		 const uschar *, int, uschar *, int);
 extern void    transport_init(void);
-extern BOOL    transport_pass_socket(const uschar *, const uschar *, const uschar *, uschar *, int
-#ifndef DISABLE_ESMTP_LIMITS
-			, unsigned, unsigned, unsigned
-#endif
-			);
 extern const uschar *transport_rcpt_address(address_item *, BOOL);
 extern BOOL    transport_set_up_command(const uschar ***, const uschar *,
 		 unsigned, int, address_item *, const uschar *, uschar **);
-extern void    transport_update_waiting(host_item *, uschar *);
+extern void    transport_update_waiting(host_item *, const uschar *);
 extern BOOL    transport_write_block(transport_ctx *, uschar *, int, BOOL);
 extern void    transport_write_reset(int);
 extern BOOL    transport_write_string(int, const char *, ...);
@@ -651,7 +665,6 @@ extern gstring * transport_show_supported(gstring *);
 extern BOOL    transport_write_message(transport_ctx *, int);
 extern void    tree_add_duplicate(const uschar *, address_item *);
 extern void    tree_add_nonrecipient(const uschar *);
-extern void    tree_add_unusable(const host_item *);
 extern void    tree_dup(tree_node **, tree_node *);
 extern int     tree_insertnode(tree_node **, tree_node *);
 extern tree_node *tree_search(tree_node *, const uschar *);
@@ -669,9 +682,9 @@ extern int     vaguely_random_number(int);
 #ifndef DISABLE_TLS
 extern int     vaguely_random_number_fallback(int);
 #endif
-extern int     verify_address(address_item *, FILE *, int, int, int, int,
+extern int     verify_address(address_item *, int, int, int, int, int,
                  uschar *, uschar *, BOOL *);
-extern int     verify_check_dnsbl(int, const uschar **, uschar **);
+extern int     verify_check_dnsbl(int, const uschar *, uschar **);
 extern int     verify_check_header_address(uschar **, uschar **, int, int, int,
                  uschar *, uschar *, int, int *);
 extern int     verify_check_headers(uschar **);
@@ -689,6 +702,7 @@ extern BOOL    verify_sender(int *, uschar **);
 extern BOOL    verify_sender_preliminary(int *, uschar **);
 extern void    version_init(void);
 
+extern BOOL    wouldblock_reading(BOOL);
 extern BOOL    write_chunk(transport_ctx *, const uschar *, int);
 extern ssize_t write_to_fd_buf(int, const uschar *, size_t);
 extern uschar *wrap_header(const uschar *, unsigned, unsigned, const uschar *, unsigned);
@@ -697,6 +711,8 @@ extern uschar *wrap_header(const uschar *, unsigned, unsigned, const uschar *, u
 extern uschar * xclient_smtp_command(uschar *, int *, BOOL *);
 extern gstring * xclient_smtp_advertise_str(gstring *);
 #endif
+extern uschar *xtextencode(const uschar *, int);
+extern int     xtextdecode(const uschar *, uschar **);
 
 
 /******************************************************************************/
@@ -758,9 +774,17 @@ if (!is_tainted(dst) && is_tainted(src)) die_tainted(US"Ustrncpy", CUS func, lin
 #endif
 return US strncpy(CS dst, CCS src, n);
 }
+#if !defined(COMPILE_UTILITY) && !defined(MACRO_PREDEF)
+static inline uschar * __Ustpcpy(uschar * dst, const uschar * src, const char * func, int line)
+{
+if (!is_tainted(dst) && is_tainted(src)) die_tainted(US"Ustpcpy", CUS func, line);
+return US stpcpy(CS dst, CCS src);
+}
+#endif
 /*XXX will likely need unchecked copy also */
 
 
+
 /* Advance the string pointer given over any whitespace.
 Return the next char as there's enough places using it to be useful. */
 
@@ -900,7 +924,7 @@ Returns:    copy of string in new store, with letters lowercased
 */
 
 static inline uschar *
-string_copynlc(uschar * s, int n)
+string_copynlc(const uschar * s, int n)
 {
 uschar * ss = store_get(n + 1, s);
 uschar * p = ss;
@@ -922,18 +946,23 @@ Returns:  copy of string in new store
 */
 
 static inline uschar *
-string_copy_perm(const uschar *s, BOOL force_taint)
+string_copy_pool(const uschar * s, BOOL force_taint, int use_pool)
 {
 int old_pool = store_pool;
 int len = Ustrlen(s) + 1;
 uschar *ss;
 
-store_pool = POOL_PERM;
+store_pool = use_pool;
 ss = store_get(len, force_taint ? GET_TAINTED : s);
 memcpy(ss, s, len);
 store_pool = old_pool;
 return ss;
 }
+
+static inline uschar *
+string_copy_perm(const uschar * s, BOOL force_taint)
+{ return string_copy_pool(s, force_taint, POOL_PERM); }
+
 # endif
 
 
@@ -1006,7 +1035,7 @@ else
 }
 
 static inline uschar *
-string_copy_from_gstring(gstring * g)
+string_copy_from_gstring(const gstring * g)
 {
 return g ? string_copyn(g->s, g->ptr) : NULL;
 }
@@ -1018,9 +1047,9 @@ return g ? (unsigned)g->ptr : 0;
 }
 
 static inline uschar
-gstring_last_char(gstring * g)
+gstring_last_char(const gstring * g)
 {
-return g->s[g->ptr-1];
+return g && g->ptr > 0 ? g->s[g->ptr-1] : '\0';
 }
 
 static inline void
@@ -1103,7 +1132,9 @@ g->s = s;
 static inline gstring *
 gstring_append(gstring * dest, gstring * item)
 {
-return string_catn(dest, item->s, item->ptr);
+return item
+  ? dest ? string_catn(dest, item->s, item->ptr) : item
+  : dest;
 }
 
 
@@ -1259,12 +1290,13 @@ timediff(diff, then);
 static inline uschar *
 string_timediff(const struct timeval * diff)
 {
-static uschar buf[sizeof("0.000s")];
+static uschar buf[16];
 
 if (diff->tv_sec >= 5 || !LOGGING(millisec))
   return readconf_printtime((int)diff->tv_sec);
 
-snprintf(CS buf, sizeof(buf), "%u.%03us", (uint)diff->tv_sec, (uint)diff->tv_usec/1000);
+snprintf(CS buf, sizeof(buf), "%u.%03us",
+	  (uint)diff->tv_sec, (uint)diff->tv_usec/1000);
 return buf;
 }
 
@@ -1298,7 +1330,7 @@ if (f.running_in_test_harness && f.testsuite_delays) millisleep(millisec);
 }
 
 /******************************************************************************/
-/* Taint-checked file opens */
+/* Taint-checked file opens. Return values/errno per open(2). */
 
 static inline int
 exim_open2(const char *pathname, int flags)
@@ -1355,23 +1387,39 @@ return NULL;
 
 /******************************************************************************/
 # if !defined(COMPILE_UTILITY)
+
+/* We use the PID of the head process for a connection-id.  Note that
+this is only for tracking a received connection and what it directly
+causes; there is no intent to describe transport-initiated TCP connections.
+The value is intented to be a cookie usable for logging, and we might change
+the generator for it at any time. */
+
+static inline void
+set_connection_id(void)
+{
+connection_id = string_sprintf("%lu", (u_long)getpid());
+}
+
+
 /* Process manipulation */
 
 static inline pid_t
 exim_fork(const unsigned char * purpose)
 {
 pid_t pid;
-DEBUG(D_any) debug_printf("%s forking for %s\n", process_purpose, purpose);
+DEBUG(D_any)
+  debug_printf_indent("%s forking for %s\n", process_purpose, purpose);
 if ((pid = fork()) == 0)
   {
   f.daemon_listen = FALSE;
   process_purpose = purpose;
-  DEBUG(D_any) debug_printf("postfork: %s\n", purpose);
+  DEBUG(D_any) debug_printf_indent("postfork: %s\n", purpose);
   }
 else
   {
   testharness_pause_ms(100); /* let child work */
-  DEBUG(D_any) debug_printf("%s forked for %s: %d\n", process_purpose, purpose, (int)pid);
+  DEBUG(D_any) debug_printf_indent("%s forked for %s: %d\n",
+				  process_purpose, purpose, (int)pid);
   }
 return pid;
 }
@@ -1404,6 +1452,19 @@ return poll(&p, 1, tmo_millisec);
 /******************************************************************************/
 /* Client-side smtp log string, for debug */
 
+static inline void
+smtp_debug_cmd_log_init(void)
+{
+#  ifndef DISABLE_CLIENT_CMD_LOG
+int old_pool = store_pool;
+store_pool = POOL_PERM;
+client_cmd_log = string_get_tainted(56, GET_TAINTED);
+*client_cmd_log->s = '\0';
+store_pool = old_pool;
+#  endif
+}
+
+
 static inline void
 smtp_debug_cmd(const uschar * buf, int mode)
 {
@@ -1412,31 +1473,46 @@ HDEBUG(D_transport|D_acl|D_v) debug_printf_indent("  SMTP%c> %s\n",
 
 #  ifndef DISABLE_CLIENT_CMD_LOG
   {
-  int len = Ustrcspn(buf, " \n");
-  int old_pool = store_pool;
+  int len = Ustrcspn(buf, " \n"), old_pool = store_pool;
   store_pool = POOL_PERM;	/* Main pool ACL allocations eg. callouts get released */
   client_cmd_log = string_append_listele_n(client_cmd_log, ':', buf, MIN(len, 8));
   if (mode == SCMD_BUFFER) 
-    {
     client_cmd_log = string_catn(client_cmd_log, US"|", 1); 
-    (void) string_from_gstring(client_cmd_log);
-    }
   else if (mode == SCMD_MORE)
-    {
     client_cmd_log = string_catn(client_cmd_log, US"+", 1);
-    (void) string_from_gstring(client_cmd_log);
-    }
   store_pool = old_pool;
   }
 #  endif
 }
 
 
+/* This might be called both due to callout and then from delivery.
+Use memory that will not be released between those phases.
+*/
+static inline void
+smtp_debug_resp(const uschar * buf)
+{
+#  ifndef DISABLE_CLIENT_CMD_LOG
+int old_pool = store_pool;
+store_pool = POOL_PERM;
+client_cmd_log = string_append_listele_n(client_cmd_log, ':', buf,
+  buf[3] == '-' ? 4 : 3);
+store_pool = old_pool;
+#  endif
+}
+
+
 static inline void
 smtp_debug_cmd_report(void)
 {
 #  ifndef DISABLE_CLIENT_CMD_LOG
-debug_printf("cmdlog: '%s'\n", client_cmd_log ? client_cmd_log->s : US"(unset)");
+if (client_cmd_log && *client_cmd_log->s)
+  {
+  debug_printf("cmdlog: '%Y'\n", client_cmd_log);
+  gstring_reset(client_cmd_log);
+  }
+else
+  debug_printf("cmdlog: (unset)\n");
 #  endif
 }
 
@@ -1445,11 +1521,22 @@ debug_printf("cmdlog: '%s'\n", client_cmd_log ? client_cmd_log->s : US"(unset)")
 static inline int
 expand_max_rcpt(const uschar * str_max_rcpt)
 {
-const uschar * s = expand_cstring(str_max_rcpt);
+const uschar * s = expand_string(str_max_rcpt);
 int res;
 return !s || !*s || (res = Uatoi(s)) == 0 ? UNLIMITED_ADDRS : res;
 }
 
+
+
+static inline void
+smtp_inout_close(void)
+{
+if (smtp_in_fd)  (void) close(smtp_in_fd);
+if (smtp_out_fd) (void) close(smtp_out_fd);
+smtp_in_fd = smtp_out_fd = -1;
+}
+
+
 /******************************************************************************/
 /* Queue-runner operations */
 
