@@ -2,11 +2,14 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
+#ifndef GLOBALS_H
+#define GLOBALS_H
+
 /* Almost all the global variables are defined together in this one header, so
 that they are easy to find. However, those that are visible during the
 compilation of the local_scan() function are defined separately in the
@@ -78,10 +81,11 @@ extern int     sqlite_lock_timeout;    /* Internal lock waiting timeout */
 extern BOOL    move_frozen_messages;   /* Get them out of the normal directory */
 #endif
 
-/* These variables are outside the #ifdef because it keeps the code less
+/* These variables are outside the TLS #ifdef because it keeps the code less
 cluttered in several places (e.g. during logging) if we can always refer to
 them. Also, the tls_ variables are now always visible. */
 
+#if !defined(MACRO_PREDEF) && !defined(COMPILE_UTILITY)
 typedef struct {
   client_conn_ctx active;     /* fd/socket when in a TLS session, and ptr to TLS context */
   int     bits;               /* bits used in TLS session */
@@ -94,8 +98,6 @@ typedef struct {
   const uschar *cipher_stdname; /* Cipher used, RFC version */
   const uschar *ver;	      /* TLS version */
 
-  BOOL    on_connect;         /* For older MTAs that don't STARTTLS */
-  uschar *on_connect_ports;   /* Ports always tls-on-connect */
   void   *ourcert;            /* Certificate we presented, binary */
   void	 *peercert;           /* Certificate of peer, binary */
   uschar *peerdn;             /* DN from peer */
@@ -116,12 +118,15 @@ typedef struct {
   BOOL	  host_resumable:1;
   BOOL	  ticket_received:1;
 #endif
-  BOOL	  verify_override:1;	/* certificate_verified only due to tls_try_verify_hosts */
+
   BOOL	  ext_master_secret:1;	/* extended-master-secret was used */
   BOOL	  channelbind_exporter:1; /* channelbinding is EXPORTER not UNIQUE */
+  BOOL    on_connect:1;         /* For older MTAs that don't STARTTLS */
+  BOOL	  verify_override:1;	/* certificate_verified only due to tls_try_verify_hosts */
 } tls_support;
 extern tls_support tls_in;
 extern tls_support tls_out;
+#endif	/*!MACRO_PREDEF*/
 
 #ifndef DISABLE_TLS
 extern BOOL    gnutls_compat_mode;     /* Less security, more compatibility */
@@ -134,10 +139,12 @@ extern uschar *tls_certificate;        /* Certificate file */
 extern uschar *tls_crl;                /* CRL File */
 extern int     tls_dh_max_bits;        /* don't accept higher lib suggestions */
 extern uschar *tls_dhparam;            /* DH param file */
+extern uschar *tls_early_banner_hosts; /* use Early Data for SMTP banner */
 extern uschar *tls_eccurve;            /* EC curve */
 # ifndef DISABLE_OCSP
 extern uschar *tls_ocsp_file;          /* OCSP stapling proof file */
 # endif
+extern uschar *tls_on_connect_ports;   /* Ports always tls-on-connect */
 extern uschar *tls_privatekey;         /* Private key file */
 extern BOOL    tls_remember_esmtp;     /* For YAEB */
 extern uschar *tls_require_ciphers;    /* So some can be avoided */
@@ -296,6 +303,7 @@ extern struct global_flags {
  BOOL   tcp_in_fastopen_data		:1; /* fastopen carried data */
  BOOL   tcp_in_fastopen_logged		:1; /* one-time logging */
  BOOL   tcp_out_fastopen_logged		:1; /* one-time logging */
+ BOOL	tls_on_connect			:1; /* all listen ports are t-o-c */
  BOOL   timestamps_utc			:1; /* Use UTC for all times */
  BOOL   transport_filter_timed_out	:1; /* True if it did */
  BOOL   trusted_caller			:1; /* Caller is trusted */
@@ -323,10 +331,11 @@ extern uschar *acl_smtp_connect;       /* ACL run on SMTP connection */
 extern uschar *acl_smtp_data;          /* ACL run after DATA received */
 #ifndef DISABLE_PRDR
 extern uschar *acl_smtp_data_prdr;     /* ACL run after DATA received if in PRDR mode*/
-extern const pcre2_code *regex_PRDR;         /* For recognizing PRDR settings */
+extern const pcre2_code *regex_PRDR;   /* For recognizing PRDR settings */
 #endif
+extern uschar *acl_smtp_atrn;          /* ACL run for ATRN */
 #ifndef DISABLE_DKIM
-extern uschar *acl_smtp_dkim;          /* ACL run for DKIM signatures / domains */
+extern uschar *acl_smtp_dkim;          /* ACL run for DKIM signatures/domains */
 #endif
 extern uschar *acl_smtp_etrn;          /* ACL run for ETRN */
 extern uschar *acl_smtp_expn;          /* ACL run for EXPN */
@@ -349,6 +358,7 @@ extern tree_node *acl_var_c;           /* ACL connection variables */
 extern tree_node *acl_var_m;           /* ACL message variables */
 extern uschar *acl_verify_message;     /* User message for verify failure */
 extern string_item *acl_warn_logged;   /* Logged lines */
+extern int acl_where;		       /* Current running ACL */
 extern uschar *acl_wherecodes[];       /* Response codes for ACL fails */
 extern uschar *acl_wherenames[];       /* Names for messages */
 extern address_item *addr_duplicate;   /* Duplicate address list */
@@ -360,21 +370,17 @@ extern int     addresslist_count;      /* Number defined */
 extern gid_t  *admin_groups;           /* List of admin groups */
 extern BOOL    allow_domain_literals;  /* As it says */
 extern BOOL    allow_mx_to_ip;         /* Allow MX records to -> ip address */
-#ifdef EXPERIMENTAL_ARC
-extern struct arc_set *arc_received;   /* highest ARC instance evaluation struct */
-extern int     arc_received_instance;  /* highest ARC instance number in headers */
-extern int     arc_oldest_pass;        /* lowest passing instance number in headers */
-extern const uschar *arc_state;	       /* verification state */
-extern const uschar *arc_state_reason;
-#endif
 extern BOOL    allow_utf8_domains;     /* For experimenting */
+extern const uschar *atrn_domains;     /* Domains requested for transfer */
+extern const uschar *atrn_host;        /* host spec for client */
+extern const uschar *atrn_mode;	       /* "C"ustomer, "P"rovider, or empty */
 extern uschar *authenticated_fail_id;  /* ID that failed authentication */
 extern uschar *authenticated_id;       /* ID that was authenticated */
 extern uschar *authenticated_sender;   /* From AUTH on MAIL */
 extern BOOL    authentication_failed;  /* TRUE if AUTH was tried and failed */
 extern uschar *authenticator_name;     /* for debug and error messages */
 extern uschar *auth_advertise_hosts;   /* Only advertise to these */
-extern auth_info auths_available[];    /* Vector of available auth mechanisms */
+extern auth_info *auths_available;	/* List of available auth mechanisms */
 extern auth_instance *auths;           /* Chain of instantiated auths */
 extern auth_instance auth_defaults;    /* Default values */
 extern uschar *auth_defer_msg;         /* Error message for log */
@@ -435,28 +441,36 @@ extern gstring *client_cmd_log;	       /* debug log of client cmds & responses *
 extern int     clmacro_count;          /* Number of command line macros */
 extern uschar *clmacros[];             /* Copy of them, for re-exec */
 extern BOOL    commandline_checks_require_admin; /* belt and braces for insecure setups */
-extern uint64_t connection_id;	       /* connection number */
+extern const uschar *connection_id;    /* connection cookie for log */
 extern int     connection_max_messages;/* Max down one SMTP connection */
 extern FILE   *config_file;            /* Configuration file */
 extern const uschar *config_filename;  /* Configuration file name */
 extern gid_t   config_gid;             /* Additional group owner */
-extern int     config_lineno;          /* Line number */
+extern unsigned config_lineno;         /* Line number */
 extern const uschar *config_main_filelist; /* List of possible config files */
 extern uschar *config_main_filename;   /* File name actually used */
 extern uschar *config_main_directory;  /* Directory where the main config file was found */
 extern uid_t   config_uid;             /* Additional owner */
-extern uschar *continue_proxy_cipher;  /* TLS cipher for proxied continued delivery */
-extern BOOL    continue_proxy_dane;    /* proxied conn is DANE */
-extern uschar *continue_proxy_sni;     /* proxied conn SNI */
-extern uschar *continue_hostname;      /* Host for continued delivery */
-extern uschar *continue_host_address;  /* IP address for ditto */
-extern int     continue_sequence;      /* Sequence num for continued delivery */
-extern uschar *continue_transport;     /* Transport for continued delivery */
+extern unsigned continue_flags;	       /* TLS-related info for connection */
 #ifndef DISABLE_ESMTP_LIMITS
 extern unsigned continue_limit_mail;   /* Peer advertised limit */
 extern unsigned continue_limit_rcpt;
 extern unsigned continue_limit_rcptdom;
 #endif
+extern int     continue_fd;	       /* Connection for continuation */
+extern uschar *continue_proxy_cipher;  /* TLS cipher for proxied continued delivery */
+extern BOOL    continue_proxy_dane;    /* proxied conn is DANE */
+extern uschar *continue_proxy_sni;     /* proxied conn SNI */
+extern const uschar *continue_hostname;      /* Host for continued delivery */
+extern const uschar *continue_host_address;  /* IP address for ditto */
+extern int     continue_host_port;     /* TCP port for ditto */
+extern uschar  continue_next_id[];     /* Next message_id from hintsdb */
+extern unsigned continue_sequence;     /* Sequence num for continued delivery */
+extern const uschar *continue_transport; /* Transport for continued delivery */
+#ifndef COMPILE_UTILITY
+extern open_db *continue_retry_db;     /* Hintsdb for retries */
+extern open_db *continue_wait_db;      /* Hintsdb for wait-transport */
+#endif
 
 
 extern uschar *csa_status;             /* Client SMTP Authorization result */
@@ -464,12 +478,13 @@ extern uschar *csa_status;             /* Client SMTP Authorization result */
 typedef struct {
   unsigned     callout_hold_only:1;    /* Conn is only for verify callout */
   unsigned     delivery:1;             /* When to attempt */
+  unsigned     tpt_sender:1;           /* Use tpt-defined sender */
   unsigned     defer_pass:1;           /* Pass 4xx to caller rather than spooling */
   unsigned     is_tls:1;	       /* Conn has TLS active */
   client_conn_ctx cctx;                /* Open connection */
   int          nrcpt;                  /* Count of addresses */
   uschar *     transport;	       /* Name of transport */
-  uschar *     interface;              /* (address of) */
+  const uschar * interface;            /* (address of) */
   uschar *     snd_ip;		       /* sending_ip_address */
   int	       snd_port;	       /* sending_port */
   unsigned     peer_options;	       /* smtp_peer_options */
@@ -498,6 +513,7 @@ extern bit_table debug_options[];      /* Table of debug options */
 extern int     debug_options_count;    /* Size of table */
 extern unsigned debug_pretrigger_bsize;
 extern uschar *debug_pretrigger_buf;   /* circular buffer for precapture */
+extern BOOL    debug_startup;	       /* Pre-config-read debugging */
 extern BOOL    debug_store;	       /* Do extra checks on store_reset */
 extern uschar  debuglog_name[LOG_NAME_SIZE]; /* ACL-init debug */
 
@@ -538,34 +554,6 @@ extern BOOL    disable_fsync;          /* Not for normal use */
 #endif
 extern BOOL    disable_ipv6;           /* Don't do any IPv6 things */
 
-#ifndef DISABLE_DKIM
-extern unsigned dkim_collect_input;    /* Runtime count of dkim signtures; tracks whether SMTP input is fed to DKIM validation */
-extern uschar *dkim_cur_signer;        /* Expansion variable, holds the current "signer" domain or identity during a acl_smtp_dkim run */
-extern int     dkim_key_length;        /* Expansion variable, length of signing key in bits */
-extern void   *dkim_signatures;	       /* Actually a (pdkim_signature *) but most files do not need to know */
-extern uschar *dkim_signers;           /* Expansion variable, holds colon-separated list of domains and identities that have signed a message */
-extern gstring *dkim_signing_record;   /* domains+selectors used */
-extern uschar *dkim_signing_domain;    /* Expansion variable, domain used for signing a message. */
-extern uschar *dkim_signing_selector;  /* Expansion variable, selector used for signing a message. */
-extern uschar *dkim_verify_hashes;     /* Preference order for signatures */
-extern uschar *dkim_verify_keytypes;   /* Preference order for signatures */
-extern uschar *dkim_verify_min_keysizes; /* list of minimum key sizes, keyed by algo */
-extern BOOL    dkim_verify_minimal;    /* Shortcircuit signature verification */
-extern uschar *dkim_verify_overall;    /* First successful domain verified, or null */
-extern uschar *dkim_verify_signers;    /* Colon-separated list of domains for each of which we call the DKIM ACL */
-extern uschar *dkim_verify_status;     /* result for this signature */
-extern uschar *dkim_verify_reason;     /* result for this signature */
-#endif
-#ifdef SUPPORT_DMARC
-extern uschar *dmarc_domain_policy;    /* Expansion for declared policy of used domain */
-extern uschar *dmarc_forensic_sender;  /* Set sender address for forensic reports */
-extern uschar *dmarc_history_file;     /* Expansion variable, file to store dmarc results */
-extern uschar *dmarc_status;           /* Expansion variable, one word value */
-extern uschar *dmarc_status_text;      /* Expansion variable, human readable value */
-extern uschar *dmarc_tld_file;         /* Mozilla TLDs text file */
-extern uschar *dmarc_used_domain;      /* Expansion variable, domain libopendmarc chose for DMARC policy lookup */
-#endif
-
 extern uschar *dns_again_means_nonexist; /* Domains that are badly set up */
 extern int     dns_csa_search_limit;   /* How deep to search for CSA SRV records */
 extern BOOL    dns_csa_use_reverse;    /* Check CSA in reverse DNS? (non-standard) */
@@ -595,6 +583,7 @@ extern int     driver_srcline;	       /* For debug & errors */
 extern unsigned int dtrigger_selector; /* when to start debug */
 
 extern uschar *dsn_from;               /* From: string for DSNs */
+extern const uschar * dsn_aware_names[]; /* Mostly for debug output */
 
 extern BOOL    envelope_to_remove;     /* Remove envelope_to_headers */
 extern int     errno_quota;            /* Quota errno in this OS */
@@ -613,7 +602,6 @@ extern const uschar *event_name;       /* event classification */
 extern gid_t   exim_gid;               /* To be used with exim_uid */
 extern BOOL    exim_gid_set;           /* TRUE if exim_gid set */
 extern uschar *exim_path;              /* Path to exec exim */
-extern const uschar *exim_sieve_extension_list[]; /* list of sieve extensions */
 extern uid_t   exim_uid;               /* Non-root uid for exim */
 extern BOOL    exim_uid_set;           /* TRUE if exim_uid set */
 extern int     expand_level;	       /* Nesting depth; indent for debug */
@@ -713,6 +701,7 @@ extern int     localpartlist_count;    /* Number defined */
 extern uschar *log_buffer;             /* For constructing log entries */
 extern int     log_default[];          /* Initialization list for log_selector */
 extern uschar *log_file_path;          /* If unset, use default */
+extern const uschar *log_ports;	       /* If set, port numbers to log */
 extern int     log_notall[];           /* Log options excluded from +all */
 extern bit_table log_options[];        /* Table of options */
 extern int     log_options_count;      /* Size of table */
@@ -722,8 +711,8 @@ extern uschar *log_selector_string;    /* As supplied in the config */
 extern FILE   *log_stderr;             /* Copy of stderr for log use, or NULL */
 extern BOOL    log_timezone;           /* TRUE to include the timezone in log lines */
 extern uschar *login_sender_address;   /* The actual sender address */
-extern lookup_info **lookup_list;      /* Array of pointers to available lookups */
-extern int     lookup_list_count;      /* Number of entries in the list */
+extern tree_node *lookups_tree;        /* Tree of available lookups */
+extern unsigned lookup_list_count;     /* Number of entries in the list */
 extern uschar *lookup_dnssec_authenticated; /* AD status of dns lookup */
 extern int     lookup_open_max;        /* Max lookup files to cache */
 extern uschar *lookup_value;           /* Value looked up from file */
@@ -833,9 +822,9 @@ extern BOOL    prod_requires_admin;    /* TRUE if prodding requires admin */
 #if defined(SUPPORT_PROXY) || defined(SUPPORT_SOCKS) || defined(EXPERIMENTAL_XCLIENT)
 extern uschar *hosts_proxy;            /* Hostlist which (require) use proxy protocol */
 extern uschar *proxy_external_address; /* IP of remote interface of proxy */
-extern int     proxy_external_port;    /* Port on remote interface of proxy */
+extern unsigned proxy_external_port;   /* Port on remote interface of proxy */
 extern uschar *proxy_local_address;    /* IP of local interface of proxy */
-extern int     proxy_local_port;       /* Port on local interface of proxy */
+extern unsigned proxy_local_port;      /* Port on local interface of proxy */
 extern int     proxy_protocol_timeout; /* Timeout for proxy negotiation */
 extern BOOL    proxy_session;          /* TRUE if receiving mail from valid proxy
 					  or sending via one */
@@ -939,10 +928,10 @@ extern int     rfc1413_query_timeout;  /* Timeout on RFC 1413 calls */
 /* extern BOOL    rfc821_domains;  */       /* If set, syntax is 821, not 822 => being abolished */
 extern uid_t   root_gid;               /* The gid for root */
 extern uid_t   root_uid;               /* The uid for root */
-extern router_info routers_available[];/* Vector of available routers */
+extern router_info *routers_available; /* List of available router drivers */
 extern router_instance *routers;       /* Chain of instantiated routers */
 extern router_instance router_defaults;/* Default values */
-extern uschar *router_name;            /* Name of router last started */
+extern const uschar *router_name;      /* Name of router last started */
 extern tree_node *router_var;	       /* Variables set by router */
 extern ip_address_item *running_interfaces; /* Host's running interfaces */
 extern uschar *running_status;         /* Flag string for testing */
@@ -1002,7 +991,7 @@ extern double  smtp_delay_rcpt;        /* Current RCPT delay */
 extern BOOL    smtp_enforce_sync;      /* Enforce sync rules */
 extern uschar *smtp_etrn_command;      /* Command to run */
 extern BOOL    smtp_etrn_serialize;    /* Only one at once */
-extern FILE   *smtp_in;                /* Incoming SMTP input file */
+extern int     smtp_in_fd;	       /* Incoming SMTP input file */
 extern int     smtp_listen_backlog;    /* Current listener socket backlog, if monitored */
 extern int     smtp_load_reserve;      /* Only from reserved if load > this */
 extern int     smtp_mailcmd_count;     /* Count of MAIL commands */
@@ -1010,12 +999,11 @@ extern int     smtp_mailcmd_max;       /* Limit for MAIL commands */
 extern int     smtp_max_synprot_errors;/* Max syntax/protocol errors */
 extern int     smtp_max_unknown_commands; /* As it says */
 extern uschar *smtp_names[];	       /* decode for command codes */
-extern uschar *smtp_notquit_reason;    /* Global for disconnect reason */
-extern FILE   *smtp_out;               /* Incoming SMTP output file */
+extern const uschar *smtp_notquit_reason; /* Global for disconnect reason */
+extern int     smtp_out_fd;	       /* Incoming SMTP output file */
 extern uschar *smtp_ratelimit_hosts;   /* Rate limit these hosts */
 extern uschar *smtp_ratelimit_mail;    /* Parameters for MAIL limiting */
 extern uschar *smtp_ratelimit_rcpt;    /* Parameters for RCPT limiting */
-extern uschar *smtp_read_error;        /* Message for SMTP input error */
 extern int     smtp_receive_timeout;   /* Applies to each received line */
 extern uschar *smtp_receive_timeout_s; /* ... expandable version */
 extern uschar *smtp_reserve_hosts;     /* Hosts for reserved slots */
@@ -1042,16 +1030,6 @@ extern uschar *spam_action;            /* the spamd recommended-action */
 extern uschar *spam_score;             /* the spam score (float) */
 extern uschar *spam_score_int;         /* spam_score * 10 (int) */
 #endif
-#ifdef SUPPORT_SPF
-extern uschar *spf_guess;              /* spf best-guess record */
-extern uschar *spf_header_comment;     /* spf header comment */
-extern uschar *spf_received;           /* Received-SPF: header */
-extern uschar *spf_result;             /* spf result in string form */
-extern BOOL    spf_result_guessed;     /* spf result is of best-guess operation */
-extern uschar *spf_smtp_comment;       /* spf comment to include in SMTP reply */
-extern uschar *spf_smtp_comment_template;
-                                       /* template to construct the spf comment by libspf2 */
-#endif
 extern BOOL    split_spool_directory;  /* TRUE to use multiple subdirs */
 extern FILE   *spool_data_file;	       /* handle for -D file */
 extern uschar *spool_directory;        /* Name of spool directory */
@@ -1087,9 +1065,7 @@ extern BOOL    system_filter_uid_set;  /* TRUE if uid set */
 extern blob    tcp_fastopen_nodata;    /* for zero-data TFO connect requests */
 extern BOOL    tcp_nodelay;            /* Controls TCP_NODELAY on daemon */
 extern tfo_state_t tcp_out_fastopen;   /* TCP fast open */
-#ifdef USE_TCP_WRAPPERS
-extern uschar *tcp_wrappers_daemon_name; /* tcpwrappers daemon lookup name */
-#endif
+extern int     test_harness_identd_port; /* For use when testing */
 extern int     test_harness_load_avg;  /* For use when testing */
 extern int     thismessage_size_limit; /* Limit for this message */
 extern int     timeout_frozen_after;   /* Max time to keep frozen messages */
@@ -1097,15 +1073,15 @@ extern int     timeout_frozen_after;   /* Max time to keep frozen messages */
 extern struct timeval timestamp_startup; /* For development measurements */
 #endif
 
-extern uschar *transport_name;         /* Name of transport last started */
+extern const uschar *transport_name;   /* Name of transport last started */
 extern int     transport_count;        /* Count of bytes transported */
 extern int     transport_newlines;     /* Accurate count of number of newline chars transported */
 extern const uschar **transport_filter_argv; /* For on-the-fly filtering */
 extern int     transport_filter_timeout; /* Timeout for same */
 
-extern transport_info transports_available[]; /* Vector of available transports */
-extern transport_instance *transports; /* Chain of instantiated transports */
-extern transport_instance transport_defaults; /* Default values */
+extern transport_info *transports_available;	/* List of available transports */
+extern transport_instance *transports;		/* Chain of instantiated transports */
+extern transport_instance transport_defaults;	/* Default values */
 
 extern int     transport_write_timeout;/* Set to time out individual writes */
 
@@ -1142,4 +1118,5 @@ extern uschar *wellknown_advertise_hosts;/* Allow WELLKNOWN command for specifie
 extern uschar *wellknown_response;     /* SMTP response for WELLKNOWN verb */
 #endif
 
+#endif	/* whole file */
 /* End of globals.h */
