@@ -1,4 +1,8 @@
-/* Copyright (c) The Exim Maintainers 2023 */
+/*************************************************
+*     Exim - an Internet mail transport agent    *
+*************************************************/
+
+/* Copyright (c) The Exim Maintainers 2023 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -56,7 +60,7 @@ slen = Ustrlen(string);
 if ((icd = iconv_open("UTF-16BE", CCS charset)) == (iconv_t)-1)
   {
   *error = string_sprintf(
-	"imapfolder: iconv_open(\"UTF-16BE\", \"%s\") failed: %s%s",
+	"imapfolder: iconv_open(\"UTF-16BE\", %q) failed: %s%s",
     charset, strerror(errno),
     errno == EINVAL ? " (maybe unsupported conversion)" : "");
   return NULL;
