@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -22,6 +22,8 @@ Coders of dlfunc routines should read the notes on tainting at the start of
 store.c
 */
 
+#ifndef LOCAL_SCAN_H
+#define LOCAL_SCAN_H
 
 /* Some basic types that make some things easier, the Exim configuration
 settings, and the store functions. */
@@ -211,7 +213,7 @@ extern void    receive_add_recipient(const uschar *, int);
 extern BOOL    receive_remove_recipient(const uschar *);
 extern uschar *rfc2047_decode(uschar *, BOOL, const uschar *, int, int *,
 			      uschar **);
-extern int     smtp_fflush(void);
+extern int     smtp_fflush(BOOL);
 extern void    smtp_printf(const char *, BOOL, ...) PRINTF_FUNCTION(1,3);
 extern void    smtp_vprintf(const char *, BOOL, va_list);
 
@@ -247,4 +249,5 @@ extern pid_t    child_open_exim2_function(int *, uschar *, uschar *, const uscha
 extern pid_t    child_open_function(uschar **, uschar **, int, int *, int *, BOOL, const uschar *);
 #endif
 
+#endif	/* whole file */
 /* End of local_scan.h */
