@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -81,7 +81,7 @@ static int type_values[] = {
 static void *
 dnsdb_open(const uschar * filename, uschar **errmsg)
 {
-return (void *)(-1);   /* Any non-0 value */
+return (void *)(1);   /* Any non-0 value */
 }
 
 
@@ -416,24 +416,25 @@ while ((domain = string_nextinlist(&keystring, &sep, NULL, 0)))
 	  continue;
 	else
 	  {
-	  uint8_t usage, selector, matching_type;
 	  uint16_t payload_length;
-	  uschar s[MAX_TLSA_EXPANDED_SIZE];
-	  uschar * sp = s;
-	  uschar * p = US rr->data;
+	  const uschar * p = US rr->data;
+	  uint8_t usage, selector, matching_type;
 
 	  usage = *p++;
 	  selector = *p++;
 	  matching_type = *p++;
 	  /* What's left after removing the first 3 bytes above */
 	  payload_length = rr->size - 3;
-	  sp += sprintf(CS s, "%d%c%d%c%d%c", usage, *outsep2,
-		  selector, *outsep2, matching_type, *outsep2);
-	  /* Now append the cert/identifier, one hex char at a time */
-	  while (payload_length-- > 0 && sp-s < (MAX_TLSA_EXPANDED_SIZE - 4))
-	    sp += sprintf(CS sp, "%02x", *p++);
 
-	  yield = string_cat(yield, s);
+	  /* Append the cert/identifier, one hex char at a time */
+
+	  if (payload_length > MAX_TLSA_EXPANDED_SIZE)
+	    payload_length = MAX_TLSA_EXPANDED_SIZE;
+	  yield = string_fmt_append(yield, "%d%c%d%c%d%c%.*H",
+				      usage, *outsep2,
+				      selector, *outsep2,
+				      matching_type, *outsep2,
+				      (int)payload_length, p);
 	  }
       else   /* T_CNAME, T_CSA, T_MX, T_MXH, T_NS, T_PTR, T_SOA, T_SRV */
         {
@@ -534,7 +535,7 @@ while ((domain = string_nextinlist(&keystring, &sep, NULL, 0)))
 	  else yield = string_cat(yield, s);
 
 	  p += rc;
-	  if (!rr_bad_increment(rr, p, 5 * sizeof(u_int32_t)))
+	  if (!rr_bad_increment(rr, p, 5 * sizeof(uint32_t)))
 	    {
 	    GETLONG(serial, p); GETLONG(refresh, p);
 	    GETLONG(retry,  p); GETLONG(expire,  p); GETLONG(minimum, p);
