@@ -2,9 +2,10 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2022 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Thanks to Paul Kelly for contributing the original code for these
 functions. */
@@ -99,7 +100,7 @@ static void
 mysql_tidy(void)
 {
 mysql_connection *cn;
-while ((cn = mysql_connections) != NULL)
+while ((cn = mysql_connections))
   {
   mysql_connections = cn->next;
   DEBUG(D_lookup) debug_printf_indent("close MYSQL connection: %s\n", cn->server);
@@ -146,27 +147,27 @@ int yield = DEFER;
 unsigned int num_fields;
 gstring * result = NULL;
 mysql_connection *cn;
-uschar *server_copy = NULL;
-uschar *sdata[4];
+uschar * server_copy = NULL;
+uschar * sdata[4];
 
 /* Disaggregate the parameters from the server argument. The order is host,
 database, user, password. We can write to the string, since it is in a
 nextinlist temporary buffer. The copy of the string that is used for caching
 has the password removed. This copy is also used for debugging output. */
 
-for (int i = 3; i > 0; i--)
+for (int j = 3; j > 0; j--)
   {
-  uschar *pp = Ustrrchr(server, '/');
+  uschar * pp = Ustrrchr(server, '/');
   if (!pp)
     {
     *errmsg = string_sprintf("incomplete MySQL server data: %s",
-      (i == 3)? server : server_copy);
+      j == 3 ? server : server_copy);
     *defer_break = TRUE;
     return DEFER;
     }
-  *pp++ = 0;
-  sdata[i] = pp;
-  if (i == 3) server_copy = string_copy(server);  /* sans password */
+  *pp++ = '\0';
+  sdata[j] = pp;
+  if (j == 3) server_copy = string_copy(server);  /* sans password */
   }
 sdata[0] = server;   /* What's left at the start */
 
@@ -206,11 +207,18 @@ if (!cn)
     *p = 0;
     }
 
-  if ((p = Ustrchr(sdata[0], ':')))
-    {
-    *p++ = 0;
-    port = Uatoi(p);
-    }
+  /* If there is a colon (":") it could be a port number after an ipv4 or
+  hostname, or could be an ipv6. The latter must have at least two colons,
+  and we look instead for a period ("."). */
+
+  if ((p = Ustrrchr(sdata[0], ':')))
+    if (  Ustrchr(sdata[0], ':') == p		/* only one colon */
+       || (p = Ustrrchr(sdata[0], '.'))		/* >1 colon, and a period */
+       )
+      {
+      *p++ = 0;
+      port = Uatoi(p);
+      }
 
   if (Ustrchr(sdata[0], '/'))
     {
@@ -256,11 +264,8 @@ if (!cn)
 
 /* Else use a previously cached connection */
 
-else
-  {
-  DEBUG(D_lookup)
-    debug_printf_indent("MYSQL using cached connection for %s\n", server_copy);
-  }
+else DEBUG(D_lookup)
+  debug_printf_indent("MYSQL using cached connection for %s\n", server_copy);
 
 /* Run the query */
 
@@ -308,14 +313,14 @@ fields = mysql_fetch_fields(mysql_result);
 
 while ((mysql_row_data = mysql_fetch_row(mysql_result)))
   {
-  unsigned long * lengths = mysql_fetch_lengths(mysql_result);
+  const unsigned long * lengths = mysql_fetch_lengths(mysql_result);
 
   if (result)
     result = string_catn(result, US"\n", 1);
 
   if (num_fields != 1)
-    for (int i = 0; i < num_fields; i++)
-      result = lf_quote(US fields[i].name, US mysql_row_data[i], lengths[i],
+    for (int j = 0; j < num_fields; j++)
+      result = lf_quote(US fields[j].name, US mysql_row_data[j], lengths[j],
 			result);
 
   else if (mysql_row_data[0] != NULL)    /* NULL value yields nothing */
@@ -432,7 +437,7 @@ while ((c = *t++))
 /* Old code:  if (count == 0) return s;
 Now always allocate and copy, to track the quoted status. */
 
-t = quoted = store_get_quoted(Ustrlen(s) + count + 1, s, idx);
+t = quoted = store_get_quoted(Ustrlen(s) + count + 1, s, idx, US"mysql");
 
 while ((c = *s++))
   {
