@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) Jeremy Harris 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -15,21 +15,25 @@ included in the main Exim build */
 
 unsigned mp_index = 0;
 
-/* Global dummy variables */
+/* Global dummy variables and functions */
 
 void fn_smtp_receive_timeout(const uschar * name, const uschar * str) {}
 uschar * syslog_facility_str;
 
+/* Solaris needs this one for the macro expand_string() */
+const uschar * expand_string_2(const uschar * string, BOOL * textonly_p)
+{return NULL; }
+
 /******************************************************************************/
 
 void
 builtin_macro_create_var(const uschar * name, const uschar * val)
 {
-printf ("static macro_item p%d = { ", mp_index);
+printf ("static macro_item p%u = { ", mp_index);
 if (mp_index == 0)
   printf(".next=NULL,");
 else
-  printf(".next=&p%d,", mp_index-1);
+  printf(".next=&p%u,", mp_index-1);
 
 printf(" .command_line=FALSE, .namelen=%d, .replen=%d,"
 	" .name=US\"%s\", .replacement=US\"%s\" };\n",
@@ -131,9 +135,6 @@ due to conflicts with other common macros. */
 #ifdef EXPAND_DLFUNC
   builtin_macro_create(US"_HAVE_DLFUNC");
 #endif
-#ifdef USE_TCP_WRAPPERS
-  builtin_macro_create(US"_HAVE_TCPWRAPPERS");
-#endif
 #ifndef DISABLE_TLS
   builtin_macro_create(US"_HAVE_TLS");
 # ifdef USE_GNUTLS
@@ -190,8 +191,11 @@ due to conflicts with other common macros. */
 #ifdef TCP_FASTOPEN
   builtin_macro_create(US"_HAVE_TCP_FASTOPEN");
 #endif
-#ifdef SUPPORT_SPF
+#ifdef EXIM_HAVE_SPF
   builtin_macro_create(US"_HAVE_SPF");
+# ifdef EXPERIMENTAL_SPF_PERL
+  builtin_macro_create(US"_HAVE_EXPERIMENTAL_SPF_PERL");
+# endif
 #endif
 #ifdef SUPPORT_SRS
   builtin_macro_create(US"_HAVE_SRS");
@@ -365,7 +369,7 @@ options();
 expansions();
 params();
 
-printf("macro_item * macros = &p%d;\n", mp_index-1);
+printf("macro_item * macros = &p%u;\n", mp_index-1);
 printf("macro_item * mlast = &p0;\n");
 exit(0);
 }
