@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -36,6 +36,16 @@ typedef struct check_address_block {
 
 
 
+static BOOL
+is_tainted_metadata(const uschar * s)
+{
+/* Not enforcing for now, only logging; will enforce in a future release */
+if (is_tainted(s))
+  log_write(0, LOG_MAIN|LOG_PANIC,
+	    "attempt to use tainted list metadata %s", s);
+return FALSE;
+}
+
 /*************************************************
 *           Generalized string match             *
 *************************************************/
@@ -96,7 +106,8 @@ check_string(void * arg, const uschar * pattern, const uschar ** valueptr,
   uschar ** error)
 {
 const check_string_block * cb = arg;
-int search_type, partial, affixlen, starflags;
+int partial, affixlen, starflags;
+const lookup_info * li;
 int expand_setup = cb->expand_setup;
 const uschar * affix, * opts;
 uschar *s;
@@ -169,6 +180,9 @@ just fall through - the match will fail. */
 
 if (cb->flags & MCS_AT_SPECIAL && pattern[0] == '@')
   {
+  if (is_tainted_metadata(pattern))
+    return DEFER;
+
   if (pattern[1] == 0)
     {
     pattern = primary_hostname;
@@ -228,7 +242,7 @@ if (cb->flags & MCS_AT_SPECIAL && pattern[0] == '@')
 
     if (rc == HOST_FIND_AGAIN)
       {
-      search_error_message = string_sprintf("DNS lookup of \"%s\" deferred", s);
+      search_error_message = string_sprintf("DNS lookup of %q deferred", s);
       return DEFER;
       }
 
@@ -268,11 +282,11 @@ if ((semicolon = Ustrchr(pattern, ';')) == NULL)
 the part of the string preceding the semicolon. */
 
 *semicolon = 0;
-search_type = search_findtype_partial(pattern, &partial, &affix, &affixlen,
+li = search_findtype_partial(pattern, &partial, &affix, &affixlen,
   &starflags, &opts);
 *semicolon = ';';
-if (search_type < 0) log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
-  search_error_message);
+if (!li)
+  log_write_die(0, LOG_MAIN, "%s", search_error_message);
 
 /* Partial matching is not appropriate for certain lookups (e.g. when looking
 up user@domain for sender rejection). There's a flag to disable it. */
@@ -281,14 +295,14 @@ if (!(cb->flags & MCS_PARTIAL)) partial = -1;
 
 /* Set the parameters for the three different kinds of lookup. */
 
-keyquery = search_args(search_type, s, semicolon+1, &filename, opts);
+keyquery = search_args(li, s, semicolon+1, &filename, opts);
 
 /* Now do the actual lookup; throw away the data returned unless it was asked
 for; partial matching is all handled inside search_find(). Note that there is
 no search_close() because of the caching arrangements. */
 
-if (!(handle = search_open(filename, search_type, 0, NULL, NULL)))
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s", search_error_message);
+if (!(handle = search_open(filename, li, 0, NULL, NULL)))
+  log_write_die(0, LOG_MAIN, "%s", search_error_message);
 result = search_find(handle, filename, keyquery, partial, affix, affixlen,
   starflags, &expand_setup, opts);
 
@@ -369,6 +383,37 @@ return US"";  /* In practice, should never happen */
 
 
 
+/* Check for a change-of-separator specification on the head of a list.
+Handle interpretation of backslash-char, in the same way that expansion would.
+
+Argument:	listp	pointer to list-pointer, updated on return to
+			next char after the spec if there is one; else unchaged
+
+Return:		separator char, or zero for no spec
+*/
+
+uschar
+matchlist_parse_sep(const uschar ** listp)
+{
+const uschar * list = *listp;
+if (Uskip_whitespace(&list) == '<')
+  {
+  const uschar * s = list+1;
+  uschar c = *s == '\\' ? string_interpret_escape(&s) : *s;
+  if (ispunct(c) || iscntrl(c))
+    {
+    DEBUG(D_lists)
+      {
+      uschar s[2] = {0}; *s = c;
+      debug_printf_indent("list separator: '%W'\n", s);
+      }
+    *listp = s+1;	/* next char after the change-of-separator */
+    return c;
+    }
+  }
+return 0;
+}
+
 /*************************************************
 *       Scan list and run matching function      *
 *************************************************/
@@ -418,7 +463,7 @@ Returns:       OK    if matched a non-negated item
 */
 
 int
-match_check_list(const uschar **listptr, int sep, tree_node **anchorptr,
+match_check_list(const uschar * const * listptr, int sep, tree_node **anchorptr,
   unsigned int **cache_ptr, int (*func)(void *,const uschar *,const uschar **,uschar **),
   void *arg, int type, const uschar *name, const uschar **valueptr)
 {
@@ -453,14 +498,17 @@ if (!*listptr)
 if the type value is greater than or equal to than MCL_NOEXPAND, do not expand
 the list. */
 
+list = *listptr;
 if (type >= MCL_NOEXPAND)
   {
-  list = *listptr;
   type -= MCL_NOEXPAND;       /* Remove the "no expand" flag */
   textonly_re = TRUE;
   }
 else
   {
+  if (sep <= 0)
+    sep = matchlist_parse_sep(&list);
+
   /* If we are searching a domain list, and $domain is not set, set it to the
   subject that is being sought for the duration of the expansion. */
 
@@ -468,26 +516,29 @@ else
     {
     check_string_block *cb = (check_string_block *)arg;
     deliver_domain = string_copy(cb->subject);
-    list = expand_string_2(*listptr, &textonly_re);
+    list = expand_string_2(list, &textonly_re);
     deliver_domain = NULL;
     }
   else
-    list = expand_string_2(*listptr, &textonly_re);
+    list = expand_string_2(list, &textonly_re);
 
   if (!list)
     {
     if (f.expand_string_forcedfail)
       {
-      HDEBUG(D_lists) debug_printf_indent("expansion of \"%s\" forced failure: "
+      HDEBUG(D_lists) debug_printf_indent("expansion of %q forced failure: "
         "assume not in this list\n", *listptr);
       return FAIL;
       }
-    log_write(0, LOG_MAIN|LOG_PANIC, "failed to expand \"%s\" while checking "
+    log_write(0, LOG_MAIN|LOG_PANIC, "failed to expand %q while checking "
       "a list: %s", *listptr, expand_string_message);
     return DEFER;
     }
   }
 
+/* If expansion had no effect on the list text, the list-test result can
+be cached */
+
 if (textonly_re) switch (type)
   {
   case MCL_STRING:
@@ -534,10 +585,12 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
     {
     if (Ustrcmp(ss, "+caseful") == 0)
       {
-      check_address_block *cb = (check_address_block *)arg;
-      uschar *at = Ustrrchr(cb->origaddress, '@');
+      check_address_block * cb = (check_address_block *)arg;
+      uschar * at;
 
-      if (at)
+      if (is_tainted_metadata(ss)) goto BAD_TAINT;
+
+      if ((at = Ustrrchr(cb->origaddress, '@')))
         Ustrncpy(cb->address, cb->origaddress, at - cb->origaddress);
       cb->flags &= ~MCS_CASELESS;
       continue;
@@ -550,7 +603,8 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
     {
     if (Ustrcmp(ss, "+caseful") == 0)
       {
-      check_string_block *cb = (check_string_block *)arg;
+      check_string_block * cb = (check_string_block *)arg;
+      if (is_tainted_metadata(ss)) goto BAD_TAINT;
       Ustrcpy(US cb->subject, cb->origsubject);
       cb->flags &= ~MCS_CASELESS;
       continue;
@@ -563,6 +617,7 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
 
   else if (type == MCL_HOST && *ss == '+')
     {
+    if (is_tainted_metadata(ss)) goto BAD_TAINT;
     if (Ustrcmp(ss, "+include_unknown") == 0)
       {
       include_unknown = TRUE;
@@ -590,7 +645,16 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
     }
 
   /* Starting with ! specifies a negative item. It is theoretically possible
-  for a local part to start with !. In that case, a regex has to be used. */
+  for a local part to start with !. In that case, a regex has to be used.
+
+  XXX It would be good to disallow a tainted ! here, but the sequence
+  "! $tainted_var" is liable to be frequently used, and requiring a
+  named-list as a workaround would mean a lot of churn. Unfortunately,
+  some attacker can feed "!badthing" into a variable that some overworked
+  admin has used in a list...
+  Maybe we could intro another meta prefix char, which does not negate the
+  element match result (but still protects against a ! in $tainted_var) ?
+  Of course, this would still require churn in configs. */
 
   if (*ss == '!')
     {
@@ -600,203 +664,10 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
   else
     yield = OK;
 
-  /* If the item does not begin with '/', it might be a + item for a named
-  list. Otherwise, it is just a single list entry that has to be matched.
-  We recognize '+' only when supplied with a tree of named lists. */
-
-  if (*ss != '/')
-    {
-    if (*ss == '+' && anchorptr)
-      {
-      int bits = 0, offset = 0, shift = 0;
-      unsigned int * use_cache_bits = original_cache_bits;
-      uschar * cached = US"";
-      namedlist_block * nb;
-      tree_node * t;
-
-      HDEBUG(D_lists)
-	{ debug_printf_indent(" start sublist %s\n", ss+1); expand_level += 2; }
-
-      if (!(t = tree_search(*anchorptr, ss+1)))
-	{
-        log_write(0, LOG_MAIN|LOG_PANIC, "unknown named%s list \"%s\"",
-          type == MCL_DOMAIN ?    " domain" :
-          type == MCL_HOST ?      " host" :
-          type == MCL_ADDRESS ?   " address" :
-          type == MCL_LOCALPART ? " local part" : "",
-          ss);
-	goto DEFER_RETURN;
-	}
-      nb = t->data.ptr;
-
-      /* If the list number is negative, it means that this list is not
-      cacheable because it contains expansion items. */
-
-      if (nb->number < 0) use_cache_bits = NULL;
-
-      /* If we have got a cache pointer, get the bits. This is not an "else"
-      because the pointer may be NULL from the start if caching is not
-      required. */
-
-      if (use_cache_bits)
-        {
-        offset = (nb->number)/16;
-        shift = ((nb->number)%16)*2;
-        bits = use_cache_bits[offset] & (3 << shift);
-        }
-
-      /* Not previously tested or no cache - run the full test */
-
-      if (bits == 0)
-        {
-        int res = match_check_list(&(nb->string), 0, anchorptr, &use_cache_bits,
-                func, arg, type, name, valueptr);
-	HDEBUG(D_lists)
-	  { expand_level -= 2; debug_printf_indent(" end sublist %s\n", ss+1); }
-
-        switch (res)
-          {
-          case OK:   bits = 1; break;
-          case FAIL: bits = 3; break;
-          case DEFER: goto DEFER_RETURN;
-          }
-
-        /* If this list was uncacheable, or a sublist turned out to be
-        uncacheable, the value of use_cache_bits will now be NULL, even if it
-        wasn't before. Ensure that this is passed up to the next level.
-        Otherwise, remember the result of the search in the cache. */
-
-        if (!use_cache_bits)
-          *cache_ptr = NULL;
-        else
-          {
-          use_cache_bits[offset] |= bits << shift;
-
-          if (valueptr)
-            {
-            int old_pool = store_pool;
-            namedlist_cacheblock *p;
-
-            /* Cached data for hosts persists over more than one message,
-            so we use the permanent store pool */
-
-            store_pool = POOL_PERM;
-            p = store_get(sizeof(namedlist_cacheblock), GET_UNTAINTED);
-            p->key = string_copy(get_check_key(arg, type));
-
-
-            p->data = *valueptr ? string_copy(*valueptr) : NULL;
-            store_pool = old_pool;
-
-            p->next = nb->cache_data;
-            nb->cache_data = p;
-            if (*valueptr)
-              HDEBUG(D_lists) debug_printf_indent("data from lookup saved for "
-                "cache for %s: key '%s' value '%s'\n", ss, p->key, *valueptr);
-            }
-          }
-        }
-
-       /* Previously cached; to find a lookup value, search a chain of values
-       and compare keys. Typically, there is only one such, but it is possible
-       for different keys to have matched the same named list. */
-
-      else
-        {
-        HDEBUG(D_lists)
-	  {
-	  expand_level -= 2;
-	  debug_printf_indent("cached %s match for %s\n",
-	    (bits & (-bits)) == bits ? "yes" : "no", ss);
-	  }
-
-        cached = US" - cached";
-        if (valueptr)
-          {
-          const uschar *key = get_check_key(arg, type);
-
-          for (namedlist_cacheblock * p = nb->cache_data; p; p = p->next)
-            if (Ustrcmp(key, p->key) == 0)
-              {
-              *valueptr = p->data;
-              break;
-              }
-          HDEBUG(D_lists) debug_printf_indent("cached lookup data = %s\n", *valueptr);
-          }
-        }
-
-      /* Result of test is indicated by value in bits. For each test, we
-      have 00 => untested, 01 => tested yes, 11 => tested no. */
-
-      if ((bits & (-bits)) == bits)    /* Only one of the two bits is set */
-        {
-        HDEBUG(D_lists) debug_printf_indent("%s %s (matched \"%s\"%s)\n", ot,
-          yield == OK ? "yes" : "no", sss, cached);
-	goto YIELD_RETURN;
-        }
-      }
-
-    /* Run the provided function to do the individual test. */
-
-    else
-      {
-      uschar * error = NULL;
-      switch ((func)(arg, ss, valueptr, &error))
-        {
-        case OK:
-	  HDEBUG(D_lists) debug_printf_indent("%s %s (matched \"%s\")\n", ot,
-	    yield == OK ? "yes" : "no", sss);
-	  goto YIELD_RETURN;
-
-        case DEFER:
-	  if (!error)
-	    error = string_sprintf("DNS lookup of \"%s\" deferred", ss);
-	  if (ignore_defer)
-	    {
-	    HDEBUG(D_lists)
-	      debug_printf_indent("%s: item ignored by +ignore_defer\n", error);
-	    break;
-	    }
-	  if (include_defer)
-	    {
-	    log_write(0, LOG_MAIN, "%s: accepted by +include_defer", error);
-	    return OK;
-	    }
-	  if (!search_error_message) search_error_message = error;
-	  goto DEFER_RETURN;
-
-        /* The ERROR return occurs when checking hosts, when either a forward
-        or reverse lookup has failed. It can also occur in a match_ip list if a
-        non-IP address item is encountered. The error string gives details of
-        which it was. */
-
-        case ERROR:
-	  if (ignore_unknown)
-	    {
-	    HDEBUG(D_lists) debug_printf_indent(
-	      "%s: item ignored by +ignore_unknown\n", error);
-	    }
-	  else
-	    {
-	    HDEBUG(D_lists) debug_printf_indent("%s %s (%s)\n", ot,
-	      include_unknown? "yes":"no", error);
-	    if (!include_unknown)
-	      {
-	      if (LOGGING(unknown_in_list))
-		log_write(0, LOG_MAIN, "list matching forced to fail: %s", error);
-	      return FAIL;
-	      }
-	    log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-	    return OK;
-	    }
-        }
-      }
-    }
-
   /* If the item is a file name, we read the file and do a match attempt
   on each line in the file, including possibly more negation processing. */
 
-  else
+  if (*ss == '/')
     {
     int file_yield = yield;       /* In case empty file */
     uschar * filename = ss;
@@ -810,9 +681,10 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
       {
       const uschar * listname = readconf_find_option(listptr);
       if (!*listname)
-        listname = string_sprintf("\"%s\"", *listptr);
-      log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s",
+        listname = string_sprintf("%q", *listptr);
+      log_write_die(0, LOG_MAIN, "%s",
         string_open_failed("%s when checking %s", sss, listname));
+      goto cppcheck_silencing;
       }
 
     /* Trailing comments are introduced by #, but in an address list or local
@@ -855,7 +727,7 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
         {
         case OK:
 	  (void)fclose(f);
-	  HDEBUG(D_lists) debug_printf_indent("%s %s (matched \"%s\" in %s)\n",
+	  HDEBUG(D_lists) debug_printf_indent("%s %s (matched %q in %s)\n",
 	    ot, yield == OK ? "yes" : "no", sss, filename);
 
 	  /* The "pattern" being matched came from the file; we use a stack-local.
@@ -913,7 +785,201 @@ while ((sss = string_nextinlist(&list, &sep, NULL, 0)))
 
     yield = file_yield;
     (void)fclose(f);
+
+    cppcheck_silencing: ;
     }
+
+  /* If the item does not begin with '/', it might be a + item for a named
+  list. Otherwise, it is just a single list entry that has to be matched.
+  We recognize '+' only when supplied with a tree of named lists. */
+
+  else if (*ss == '+' && anchorptr)
+    {
+    int bits = 0, offset = 0, shift = 0;
+    unsigned int * use_cache_bits = original_cache_bits;
+    uschar * cached = US"";
+    namedlist_block * nb;
+    tree_node * t;
+
+    HDEBUG(D_lists)
+      { debug_printf_indent(" start sublist %s\n", ss+1); expand_level += 2; }
+
+    if (is_tainted_metadata(ss)) goto BAD_TAINT;
+    if (!(t = tree_search(*anchorptr, ss+1)))
+      {
+      log_write(0, LOG_MAIN|LOG_PANIC, "unknown named%s list %q",
+	type == MCL_DOMAIN ?    " domain" :
+	type == MCL_HOST ?      " host" :
+	type == MCL_ADDRESS ?   " address" :
+	type == MCL_LOCALPART ? " local part" : "",
+	ss);
+      goto DEFER_RETURN;
+      }
+    nb = t->data.ptr;
+
+    /* If the list number is negative, it means that this list is not
+    cacheable because it contains expansion items. */
+
+    if (nb->number < 0) use_cache_bits = NULL;
+
+    /* If we have got a cache pointer, get the bits. This is not an "else"
+    because the pointer may be NULL from the start if caching is not
+    required. */
+
+    if (use_cache_bits)
+      {
+      offset = (nb->number)/16;
+      shift = ((nb->number)%16)*2;
+      bits = use_cache_bits[offset] & (3 << shift);
+      }
+
+    /* Not previously tested or no cache - run the full test */
+
+    if (bits == 0)
+      {
+      int res = match_check_list(&(nb->string), 0, anchorptr, &use_cache_bits,
+	      func, arg, type, name, valueptr);
+      HDEBUG(D_lists)
+	{ expand_level -= 2; debug_printf_indent(" end sublist %s\n", ss+1); }
+
+      switch (res)
+	{
+	case OK:   bits = 1; break;
+	case FAIL: bits = 3; break;
+	case DEFER: goto DEFER_RETURN;
+	}
+
+      /* If this list was uncacheable, or a sublist turned out to be
+      uncacheable, the value of use_cache_bits will now be NULL, even if it
+      wasn't before. Ensure that this is passed up to the next level.
+      Otherwise, remember the result of the search in the cache. */
+
+      if (!use_cache_bits)
+	*cache_ptr = NULL;
+      else
+	{
+	use_cache_bits[offset] |= bits << shift;
+
+	if (valueptr)
+	  {
+	  int old_pool = store_pool;
+	  namedlist_cacheblock *p;
+
+	  /* Cached data for hosts persists over more than one message,
+	  so we use the permanent store pool */
+
+	  store_pool = POOL_PERM;
+	  p = store_get(sizeof(namedlist_cacheblock), GET_UNTAINTED);
+	  p->key = string_copy(get_check_key(arg, type));
+
+
+	  p->data = *valueptr ? string_copy(*valueptr) : NULL;
+	  store_pool = old_pool;
+
+	  p->next = nb->cache_data;
+	  nb->cache_data = p;
+	  if (*valueptr)
+	    HDEBUG(D_lists) debug_printf_indent("data from lookup saved for "
+	      "cache for %s: key '%s' value '%s'\n", ss, p->key, *valueptr);
+	  }
+	}
+      }
+
+     /* Previously cached; to find a lookup value, search a chain of values
+     and compare keys. Typically, there is only one such, but it is possible
+     for different keys to have matched the same named list. */
+
+    else
+      {
+      HDEBUG(D_lists)
+	{
+	expand_level -= 2;
+	debug_printf_indent("cached %s match for %s\n",
+	  (bits & (-bits)) == bits ? "yes" : "no", ss);
+	}
+
+      cached = US" - cached";
+      if (valueptr)
+	{
+	const uschar *key = get_check_key(arg, type);
+
+	for (namedlist_cacheblock * p = nb->cache_data; p; p = p->next)
+	  if (Ustrcmp(key, p->key) == 0)
+	    {
+	    *valueptr = p->data;
+	    break;
+	    }
+	HDEBUG(D_lists) debug_printf_indent("cached lookup data = %s\n", *valueptr);
+	}
+      }
+
+    /* Result of test is indicated by value in bits. For each test, we
+    have 00 => untested, 01 => tested yes, 11 => tested no. */
+
+    if ((bits & (-bits)) == bits)    /* Only one of the two bits is set */
+      {
+      HDEBUG(D_lists) debug_printf_indent("%s %s (matched %q%s)\n", ot,
+	yield == OK ? "yes" : "no", sss, cached);
+      goto YIELD_RETURN;
+      }
+    }
+
+  /* Run the provided function to do the individual test. */
+
+  else
+    {
+    uschar * error = NULL;
+    switch ((func)(arg, ss, valueptr, &error))
+      {
+      case OK:
+	HDEBUG(D_lists) debug_printf_indent("%s %s (matched %q)\n", ot,
+	  yield == OK ? "yes" : "no", sss);
+	goto YIELD_RETURN;
+
+      case DEFER:
+	if (!error)
+	  error = string_sprintf("DNS lookup of %q deferred", ss);
+	if (ignore_defer)
+	  {
+	  HDEBUG(D_lists)
+	    debug_printf_indent("%s: item ignored by +ignore_defer\n", error);
+	  break;
+	  }
+	if (include_defer)
+	  {
+	  log_write(0, LOG_MAIN, "%s: accepted by +include_defer", error);
+	  return OK;
+	  }
+	if (!search_error_message) search_error_message = error;
+	goto DEFER_RETURN;
+
+      /* The ERROR return occurs when checking hosts, when either a forward
+      or reverse lookup has failed. It can also occur in a match_ip list if a
+      non-IP address item is encountered. The error string gives details of
+      which it was. */
+
+      case ERROR:
+	if (ignore_unknown)
+	  {
+	  HDEBUG(D_lists) debug_printf_indent(
+	    "%s: item ignored by +ignore_unknown\n", error);
+	  }
+	else
+	  {
+	  HDEBUG(D_lists) debug_printf_indent("%s %s (%s)\n", ot,
+	    include_unknown? "yes":"no", error);
+	  if (!include_unknown)
+	    {
+	    if (LOGGING(unknown_in_list))
+	      log_write(0, LOG_MAIN, "list matching forced to fail: %s", error);
+	    return FAIL;
+	    }
+	  log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+	  return OK;
+	  }
+      }
+    }
+
   }    /* Loop for the next item on the top-level list */
 
 /* End of list reached: if the last item was negated yield OK, else FAIL. */
@@ -927,6 +993,7 @@ return yield == OK ? FAIL : OK;
  
 /* Something deferred */
 
+BAD_TAINT:
 DEFER_RETURN:
   HDEBUG(D_any)
     {
@@ -964,7 +1031,7 @@ recursion.
 
 Arguments:
   s              string to search for
-  listptr        ptr to ptr to colon separated list of patterns, or NULL
+  listptr        ptr to ptr to (default colon sep) list of patterns, or NULL
   sep            a separator value for the list (see string_nextinlist())
 		 or zero for auto
   anchorptr      ptr to tree for named items, or NULL if no named items
@@ -989,7 +1056,7 @@ Returns:         OK    if matched a non-negated item
 */
 
 int
-match_isinlist(const uschar *s, const uschar **listptr, int sep,
+match_isinlist(const uschar * s, const uschar * const * listptr, int sep,
    tree_node **anchorptr,
   unsigned int *cache_bits, int type, BOOL caseless, const uschar **valueptr)
 {
@@ -1067,7 +1134,7 @@ empty. Otherwise, a subject with no domain is a serious configuration error. */
 if (!sdomain && *subject)
   {
   log_write(0, LOG_MAIN|LOG_PANIC, "no @ found in the subject of an "
-    "address list match: subject=\"%s\" pattern=\"%s\"", subject, pattern);
+    "address list match: subject=%q pattern=%q", subject, pattern);
   return FAIL;
   }
 
@@ -1094,7 +1161,7 @@ if (*s == ';')
   {
   if (Ustrncmp(pattern, "partial-", 8) == 0)
     log_write(0, LOG_MAIN|LOG_PANIC, "partial matching is not applicable to "
-      "whole-address lookups: ignored \"partial-\" in \"%s\"", pattern);
+      "whole-address lookups: ignored \"partial-\" in %q", pattern);
   return match_check_string(subject, pattern, -1, cb->flags, valueptr);
   }
 
@@ -1131,8 +1198,7 @@ if (pattern[0] == '@' && pattern[1] == '@')
 
     ss = Ustrrchr(list, ':');
     if (!ss) ss = list; else ss++;
-    Uskip_whitespace(&ss);
-    if (*ss == '>')
+    if (Uskip_whitespace(&ss) == '>')
       {
       *ss++ = 0;
       Uskip_whitespace(&ss);
@@ -1172,7 +1238,7 @@ if (pattern[0] == '@' && pattern[1] == '@')
   /* End of chain loop; panic if too many times */
 
   if (watchdog <= 0)
-    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "Loop detected in lookup of "
+    log_write_die(0, LOG_MAIN, "Loop detected in lookup of "
       "local part of %s in %s", subject, pattern);
 
   /* Otherwise the local part check has failed, so the whole match
@@ -1196,12 +1262,12 @@ if ((pdomain = Ustrrchr(pattern, '@')))
   automatically interpreted in match_check_string. We just need to arrange that
   the leading @ is included in the domain. */
 
-  if (pdomain > pattern && pdomain[-1] == '@' &&
-       (pdomain[1] == 0 ||
-        Ustrcmp(pdomain+1, "[]") == 0 ||
-        Ustrcmp(pdomain+1, "mx_any") == 0 ||
-        Ustrcmp(pdomain+1, "mx_primary") == 0 ||
-        Ustrcmp(pdomain+1, "mx_secondary") == 0))
+  if (pdomain > pattern && pdomain[-1] == '@'
+     && (pdomain[1] == 0
+        || Ustrcmp(pdomain+1, "[]") == 0
+        || Ustrcmp(pdomain+1, "mx_any") == 0
+        || Ustrcmp(pdomain+1, "mx_primary") == 0
+        || Ustrcmp(pdomain+1, "mx_secondary") == 0))
     pdomain--;
 
   pllen = pdomain - pattern;
