@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -28,11 +28,9 @@ Returns:   boolean for Proxy Protocol needed
 BOOL
 proxy_protocol_host(void)
 {
-int rc;
-
 if (  sender_host_address
-   && (rc = verify_check_this_host(CUSS &hosts_proxy, NULL, NULL,
-                           sender_host_address, NULL)) == OK)
+   && verify_check_this_host(CUSS &hosts_proxy, NULL, NULL,
+                           sender_host_address, NULL) == OK)
   {
   DEBUG(D_receive)
     debug_printf("Detected proxy protocol configured host\n");
@@ -64,19 +62,17 @@ Return the amount read.
 */
 
 static int
-swallow_until_crlf(int fd, uschar *base, int already, int capacity)
+swallow_until_crlf(int fd, uschar * base, int already, int capacity)
 {
-uschar *to = base + already;
-uschar *cr;
-int have = 0;
-int ret;
-int last = 0;
+uschar * to = base + already;
+const uschar * cr;
+int have = 0, ret;
+BOOL last = FALSE;
 
 /* For "PROXY UNKNOWN\r\n" we, at time of writing, expect to have read
 up through the \r; for the _normal_ case, we haven't yet seen the \r. */
 
-cr = memchr(base, '\r', already);
-if (cr != NULL)
+if ((cr = memchr(base, '\r', already)))
   {
   if ((cr - base) < already - 1)
     {
@@ -85,19 +81,22 @@ if (cr != NULL)
     return 0;
     }
   /* \r is last character read, just need one more. */
-  last = 1;
+  last = TRUE;
   }
 
 while (capacity > 0)
   {
-  do { ret = read(fd, to, 1); } while (ret == -1 && errno == EINTR && !had_command_timeout);
+  do
+    { ret = read(fd, to, 1); }
+  while (ret == -1 && errno == EINTR && !had_command_timeout);
+
   if (ret == -1)
     return -1;
   have++;
   if (last)
     return have;
   if (*to == '\r')
-    last = 1;
+    last = TRUE;
   capacity--;
   to++;
   }
@@ -111,9 +110,7 @@ return -1;
 static void
 proxy_debug(uschar * buf, unsigned start, unsigned end)
 {
-debug_printf("PROXY<<");
-while (start < end) debug_printf(" %02x", buf[start++]);
-debug_printf("\n");
+debug_printf("PROXY<<%3.*H\n", (int)(end - start), buf + start);
 }
 
 
@@ -199,12 +196,9 @@ we have to do a minimum of 3 read calls, not 1.  Eww.
 #  error Code bug in sizes of data to read for proxy usage
 # endif
 
-int get_ok = 0;
 int size, ret;
-int fd = fileno(smtp_in);
 const char v2sig[12] = "\x0D\x0A\x0D\x0A\x00\x0D\x0A\x51\x55\x49\x54\x0A";
 uschar * iptype;  /* To display debug info */
-socklen_t vslen = sizeof(struct timeval);
 BOOL yield = FALSE;
 
 ALARM(proxy_protocol_timeout);
@@ -215,7 +209,7 @@ do
   don't do a PEEK into the data, actually slurp up enough to be
   "safe". Can't take it all because TLS-on-connect clients follow
   immediately with TLS handshake. */
-  ret = read(fd, &hdr, PROXY_INITIAL_READ);
+  ret = read(smtp_in_fd, &hdr, PROXY_INITIAL_READ);
   } while (ret == -1 && errno == EINTR && !had_command_timeout);
 
 if (ret == -1)
@@ -233,7 +227,7 @@ if ((ret == PROXY_INITIAL_READ) && (memcmp(&hdr.v2, v2sig, sizeof(v2sig)) == 0))
   /* First get the length fields. */
   do
     {
-    retmore = read(fd, (uschar*)&hdr + ret, PROXY_V2_HEADER_SIZE - PROXY_INITIAL_READ);
+    retmore = read(smtp_in_fd, US &hdr + ret, PROXY_V2_HEADER_SIZE - PROXY_INITIAL_READ);
     } while (retmore == -1 && errno == EINTR && !had_command_timeout);
   if (retmore == -1)
     goto proxyfail;
@@ -272,7 +266,7 @@ if ((ret == PROXY_INITIAL_READ) && (memcmp(&hdr.v2, v2sig, sizeof(v2sig)) == 0))
     {
     do
       {
-      retmore = read(fd, (uschar*)&hdr + ret, size-ret);
+      retmore = read(smtp_in_fd, US &hdr + ret, size-ret);
       } while (retmore == -1 && errno == EINTR && !had_command_timeout);
     if (retmore == -1)
       goto proxyfail;
@@ -376,7 +370,7 @@ else if (ret >= 8 && memcmp(hdr.v1.line, "PROXY", 5) == 0)
   char   *endc;
 
   /* get the rest of the line */
-  r2 = swallow_until_crlf(fd, (uschar*)&hdr, ret, sizeof(hdr)-ret);
+  r2 = swallow_until_crlf(smtp_in_fd, US &hdr, ret, sizeof(hdr)-ret);
   if (r2 == -1)
     goto proxyfail;
   ret += r2;
@@ -384,7 +378,7 @@ else if (ret >= 8 && memcmp(hdr.v1.line, "PROXY", 5) == 0)
   p = string_copy(hdr.v1.line);
   end = memchr(p, '\r', ret - 1);
 
-  if (!end || (end == (uschar*)&hdr + ret) || end[1] != '\n')
+  if (!end || (end == US &hdr + ret) || end[1] != '\n')
     {
     DEBUG(D_receive) debug_printf("Partial or invalid PROXY header\n");
     goto proxyfail;
@@ -489,7 +483,7 @@ else
   {
   /* Wrong protocol */
   DEBUG(D_receive) debug_printf("Invalid proxy protocol version negotiation\n");
-  (void) swallow_until_crlf(fd, (uschar*)&hdr, ret, sizeof(hdr)-ret);
+  (void) swallow_until_crlf(smtp_in_fd, US &hdr, ret, sizeof(hdr)-ret);
   goto proxyfail;
   }
 
@@ -505,7 +499,6 @@ proxyfail:
   DEBUG(D_receive) if (had_command_timeout)
     debug_printf("Timeout while reading proxy header\n");
 
-bad:
   if (yield)
     {
     sender_host_name = NULL;
