@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -41,9 +41,9 @@ rf_get_errors_address(address_item * addr, router_instance * rblock,
 uschar * s;
 
 *errors_to = addr->prop.errors_address;
+GET_OPTION("errors_to");
 if (!rblock->errors_to) return OK;
 
-GET_OPTION("errors_to");
 if (!(s = expand_string(rblock->errors_to)))
   {
   if (f.expand_string_forcedfail)
@@ -52,8 +52,8 @@ if (!(s = expand_string(rblock->errors_to)))
       debug_printf("forced expansion failure - ignoring errors_to\n");
     return OK;
     }
-  addr->message = string_sprintf("%s router failed to expand \"%s\": %s",
-    rblock->name, rblock->errors_to, expand_string_message);
+  addr->message = string_sprintf("%s router failed to expand %q: %s",
+    rblock->drinst.name, rblock->errors_to, expand_string_message);
   return DEFER;
   }
 
@@ -109,7 +109,7 @@ else
 
   DEBUG(D_route|D_verify)
     debug_printf("------ Verifying errors address %s ------\n", s);
-  if (verify_address(snew, NULL,
+  if (verify_address(snew, -1,
       vopt_is_recipient /* vopt_fake_sender is the alternative */
       | vopt_qualify, -1, -1, -1, NULL, NULL, NULL) == OK)
     *errors_to = snew->address;
