@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2021 - 2022 */
+/* Copyright (c) The Exim Maintainers 2021 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -36,6 +36,7 @@ rf_get_munge_headers(address_item *addr, router_instance *rblock,
 /* Default is to retain existing headers */
 *extra_headers = addr->prop.extra_headers;
 
+GET_OPTION("headers_add");
 if (rblock->extra_headers)
   {
   const uschar * list = rblock->extra_headers;
@@ -49,8 +50,8 @@ if (rblock->extra_headers)
       if (!f.expand_string_forcedfail)
 	{
 	addr->message = string_sprintf(
-	  "%s router failed to expand add_headers item \"%s\": %s",
-	  rblock->name, t, expand_string_message);
+	  "%s router failed to expand add_headers item %q: %s",
+	  rblock->drinst.name, t, expand_string_message);
 	return DEFER;
 	}
       }
@@ -88,6 +89,7 @@ if (rblock->extra_headers)
 *remove_headers = addr->prop.remove_headers;
 
 /* Expand items from colon-sep list separately, then build new list */
+GET_OPTION("headers_remove");
 if (rblock->remove_headers)
   {
   const uschar * list = rblock->remove_headers;
@@ -104,8 +106,8 @@ if (rblock->remove_headers)
       if (!f.expand_string_forcedfail)
 	{
 	addr->message = string_sprintf(
-	  "%s router failed to expand remove_headers item \"%s\": %s",
-	  rblock->name, t, expand_string_message);
+	  "%s router failed to expand remove_headers item %q: %s",
+	  rblock->drinst.name, t, expand_string_message);
 	return DEFER;
 	}
       }
