@@ -2,10 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Michael Haardt 2015
+/*
+ * Copyright (c) The Exim Maintainers 2016 - 2025
  * Copyright (c) Jeremy Harris 2015 - 2016
- * Copyright (c) The Exim Maintainers 2016 */
-/* See the file NOTICE for conditions of use and distribution. */
+ * Copyright (c) Michael Haardt 2015
+ * See the file NOTICE for conditions of use and distribution.
+ */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* This module provides (un)setenv routines for those environments
 lacking them in libraries. It is #include'd by OS/os.c-foo files. */
@@ -14,7 +17,6 @@ lacking them in libraries. It is #include'd by OS/os.c-foo files. */
 int
 setenv(const char * name, const char * val, int overwrite)
 {
-uschar * s;
 if (Ustrchr(name, '=')) return -1;
 if (overwrite || !getenv(name))
   putenv(CS string_copy_perm(string_sprintf("%s=%s", name, val), FALSE));
