@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -52,7 +52,7 @@ if (f) (void)fclose(f);
 if (errmsg)
   *errmsg = msg;
 else
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s", msg);
+  log_write_die(0, LOG_MAIN, "%s", msg);
 
 return -1;
 }
@@ -124,9 +124,11 @@ spool_var_write(FILE * fp, const uschar * name, const uschar * val)
 putc('-', fp);
 if (is_tainted(val))
   {
-  int q = quoter_for_address(val);
+  const uschar * quoter_name;
   putc('-', fp);
-  if (is_real_quoter(q)) fprintf(fp, "(%s)", lookup_list[q]->name);
+  (void) quoter_for_address(val, &quoter_name);
+  if (quoter_name)
+    fprintf(fp, "(%s)", quoter_name);
   }
 fprintf(fp, "%s %s\n", name, val);
 }
@@ -464,7 +466,7 @@ uschar * fname = spool_fname(string_sprintf("%s%s", from, dir), subdir, id, suff
 uschar * tname = spool_q_fname(string_sprintf("%s%s", to,   dir), dq, subdir, id, suffix);
 if (Ulink(fname, tname) < 0 && (!noentok || errno != ENOENT))
   {
-  log_write(0, LOG_MAIN|LOG_PANIC, "link(\"%s\", \"%s\") failed while moving "
+  log_write(0, LOG_MAIN|LOG_PANIC, "link(%q, %q) failed while moving "
     "message: %s", fname, tname, strerror(errno));
   return FALSE;
   }
@@ -500,7 +502,7 @@ break_link(const uschar * dir, const uschar * subdir, const uschar * id,
 uschar * fname = spool_fname(string_sprintf("%s%s", from, dir), subdir, id, suffix);
 if (Uunlink(fname) < 0 && (!noentok || errno != ENOENT))
   {
-  log_write(0, LOG_MAIN|LOG_PANIC, "unlink(\"%s\") failed while moving "
+  log_write(0, LOG_MAIN|LOG_PANIC, "unlink(%q) failed while moving "
     "message: %s", fname, strerror(errno));
   return FALSE;
   }
