@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -294,7 +294,7 @@ r = store_mark();
 if (list)
   {
   int sep = 0;
-  for (uschar * s; s = string_nextinlist(&filename, &sep, NULL, 0); )
+  for (const uschar * s; s = string_nextinlist(&filename, &sep, NULL, 0); )
     if (!(rc = tls_set_one_watch(s))) break;
   }
 else
@@ -310,13 +310,14 @@ void
 tls_watch_discard_event(int fd)
 {
 #ifdef EXIM_HAVE_INOTIFY
-(void) read(fd, big_buffer, big_buffer_size);
+int rc = read(fd, big_buffer, big_buffer_size);
 #endif
 #ifdef EXIM_HAVE_KEVENT
 struct kevent kev;
 struct timespec t = {0};
-(void) kevent(fd, NULL, 0, &kev, 1, &t);
+int rc = kevent(fd, NULL, 0, &kev, 1, &t);
 #endif
+rc = rc;	/* stupid compiler quietening */
 }
 #endif	/*EXIM_HAVE_INOTIFY*/
 
@@ -324,8 +325,8 @@ struct timespec t = {0};
 void
 tls_client_creds_reload(BOOL watch)
 {
-for(transport_instance * t = transports; t; t = t->next)
-  if (Ustrcmp(t->driver_name, "smtp") == 0)
+for(transport_instance * t = transports; t; t = t->drinst.next)
+  if (Ustrcmp(t->drinst.driver_name, "smtp") == 0)
     {
 #if defined(EXIM_HAVE_INOTIFY) || defined(EXIM_HAVE_KEVENT)
     tls_client_creds_invalidate(t);
@@ -465,8 +466,8 @@ tzset();
 *        Many functions are package-specific     *
 *************************************************/
 /* Forward decl. */
-static void tls_client_resmption_key(tls_support *, smtp_connect_args *,
-  smtp_transport_options_block *);
+static void tls_client_resmption_key(tls_support *, const smtp_connect_args *,
+  const smtp_transport_options_block *);
 
 
 #ifdef USE_GNUTLS
@@ -504,7 +505,7 @@ int
 tls_ungetc(int ch)
 {
 if (ssl_xfer_buffer_lwm <= 0)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "buffer underflow in tls_ungetc");
+  log_write_die(0, LOG_MAIN, "buffer underflow in tls_ungetc");
 
 ssl_xfer_buffer[--ssl_xfer_buffer_lwm] = ch;
 return ch;
@@ -623,8 +624,7 @@ tls_field_from_dn(uschar * dn, const uschar * mod)
 {
 int insep = ',';
 uschar outsep = '\n';
-uschar * ele;
-uschar * match = NULL;
+const uschar * match = NULL, * ele;
 int len;
 gstring * list = NULL;
 
@@ -654,7 +654,7 @@ Return TRUE for a match
 static BOOL
 is_name_match(const uschar * name, const uschar * pat)
 {
-uschar * cp;
+const uschar * cp;
 return *pat == '*'		/* possible wildcard match */
   ?    *++pat == '.'		/* starts star, dot              */
     && !Ustrchr(++pat, '*')	/* has no more stars             */
@@ -778,7 +778,7 @@ Returns:  bool for "okay"; false will cause caller to immediately exit.
 BOOL
 tls_dropprivs_validate_require_cipher(BOOL nowarn)
 {
-const uschar *errmsg;
+const uschar * errmsg;
 pid_t pid;
 int rc, status;
 void (*oldsignal)(int);
@@ -799,7 +799,7 @@ oldsignal = signal(SIGCHLD, SIG_DFL);
 
 fflush(NULL);
 if ((pid = exim_fork(US"cipher-validate")) < 0)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "fork failed for TLS check");
+  log_write_die(0, LOG_MAIN, "fork failed for TLS check");
 
 if (pid == 0)
   {
@@ -809,7 +809,7 @@ if (pid == 0)
         US"calling tls_validate_require_cipher");
 
   if ((errmsg = tls_validate_require_cipher()))
-    log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+    log_write_die(0, LOG_CONFIG,
         "tls_require_ciphers invalid: %s", errmsg);
   fflush(NULL);
   exim_underbar_exit(EXIT_SUCCESS);
@@ -832,13 +832,12 @@ return status == 0;
 
 
 static void
-tls_client_resmption_key(tls_support * tlsp, smtp_connect_args * conn_args,
-  smtp_transport_options_block * ob)
+tls_client_resmption_key(tls_support * tlsp,
+  const smtp_connect_args * conn_args, const smtp_transport_options_block * ob)
 {
 #ifndef DISABLE_TLS_RESUME
 hctx * h = &tlsp->resume_hctx;
 blob b;
-gstring * g;
 
 DEBUG(D_tls) if (conn_args->host_lbserver)
   debug_printf("TLS: lbserver '%s'\n", conn_args->host_lbserver);
@@ -863,9 +862,7 @@ exim_sha_update_string(h, tlsp->sni);
 exim_sha_update_string(h, ob->tls_alpn);
 # endif
 exim_sha_finish(h, &b);
-for (g = string_get(b.len*2+1); b.len-- > 0; )
-  g = string_fmt_append(g, "%02x", *b.data++);
-tlsp->resume_index = string_from_gstring(g);
+tlsp->resume_index = string_sprintf("%.*H", (int)b.len, b.data);
 DEBUG(D_tls) debug_printf("TLS: resume session index %s\n", tlsp->resume_index);
 #endif
 }
@@ -905,7 +902,7 @@ if (*sni)
   conn_args.tblock = tb = store_get(sizeof(*tb), GET_UNTAINTED);
   memset(tb, 0, sizeof(*tb));
 
-  tb->options_block = ob = store_get(sizeof(*ob), GET_UNTAINTED);
+  tb->drinst.options_block = ob = store_get(sizeof(*ob), GET_UNTAINTED);
   memcpy(ob, &smtp_transport_option_defaults, sizeof(*ob));
 
   ob->tls_sni = sni;
