@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
+/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -95,6 +95,7 @@ typedef struct {
   BOOL		delay_after_cutoff;
   BOOL		hosts_override;
   BOOL		hosts_randomize;
+  uschar *	expand_hosts_randomize;
   BOOL		keepalive;
   BOOL		lmtp_ignore_quota;
   uschar	*expand_retry_include_ip_address;
@@ -149,6 +150,10 @@ typedef struct {
   BOOL verify:1;
   BOOL lmtp:1;
   BOOL smtps:1;
+#ifdef EXPERIMENTAL_SRV_SMTPS
+  BOOL require_tls:1;
+#endif
+
   BOOL ok:1;
   BOOL setting_up:1;
 #ifndef DISABLE_PIPE_CONNECT
@@ -166,7 +171,7 @@ typedef struct {
 #endif
   BOOL dsn_all_lasthop:1;
 #if !defined(DISABLE_TLS) && defined(SUPPORT_DANE)
-  BOOL dane_required:1;
+  BOOL dane_required:1;		/* required by transport option */
 #endif
 #ifndef DISABLE_PIPE_CONNECT
   BOOL pending_BANNER:1;
@@ -240,15 +245,14 @@ extern smtp_transport_options_block smtp_transport_option_defaults;
 
 /* The main, init, and closedown entry points for the transport */
 
+extern void smtp_transport_init(driver_instance *);
 extern BOOL smtp_transport_entry(transport_instance *, address_item *);
-extern void smtp_transport_init(transport_instance *);
 extern void smtp_transport_closedown(transport_instance *);
 
 
 
 #ifdef SUPPORT_SOCKS
-extern int     socks_sock_connect(host_item *, int, int, uschar *,
-	         transport_instance *, int);
+extern int     socks_sock_connect(smtp_connect_args *, const blob *);
 #endif
 
 /* End of transports/smtp.h */
