@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2023 - 2024 */
+/* Copyright (c) The Exim Maintainers 2023 - 2025 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
@@ -10,7 +10,9 @@
 
 #ifdef EXPERIMENTAL_XCLIENT
 
-/* From https://www.postfix.org/XCLIENT_README.html I infer two generations of
+/* This is a proxy protocol.
+
+From https://www.postfix.org/XCLIENT_README.html I infer two generations of
 protocol.  The more recent one obviates the utility of the HELO attribute, since
 it mandates the proxy always sending a HELO/EHLO smtp command following (a
 successful) XCLIENT command, and that will carry a NELO name (which we assume,
@@ -64,9 +66,9 @@ Returns:      the number of bytes in the result, excluding the final zero;
 */
 
 static int
-xclient_xtextdecode(uschar * code, uschar * end, uschar ** ptr)
+xclient_xtextdecode(const uschar * code, const uschar * end, uschar ** ptr)
 {
-return auth_xtextdecode(string_copyn(code, end-code), ptr);
+return xtextdecode(string_copyn(code, end-code), ptr);
 }
 
 /*************************************************
@@ -78,7 +80,7 @@ return auth_xtextdecode(string_copyn(code, end-code), ptr);
 Arguments:
   s       	the data portion of the line (already past any white space)
   resp		result: smtp respose code
-  flag		input: helo seen  output: fail is fatal
+  flagp		input: helo seen  output: fail is fatal
 
 Return: NULL on success, or error message
 */
@@ -87,7 +89,7 @@ Return: NULL on success, or error message
 # define XCLIENT_TEMPUNAVAIL US"[TEMPUNAVAIL]"
 
 uschar *
-xclient_smtp_command(uschar * s, int * resp, BOOL * flag)
+xclient_smtp_command(uschar * s, int * resp, BOOL * flagp)
 {
 uschar * word = s;
 enum {
@@ -97,11 +99,11 @@ enum {
 } state = XCLIENT_SKIP_SPACES;
 enum xclient_cmd_e cmd;
 
-if (  !flag
+if (  !*flagp
    && verify_check_host(&hosts_require_helo) == OK)
   {
   *resp = 503;
-  *flag = FALSE;
+  *flagp= FALSE;
   return US"no HELO/EHLO given";
   }
 
@@ -112,14 +114,14 @@ session to avoid mixups. */
 if(!proxy_session && verify_check_host(&hosts_xclient) == FAIL)
   {
   *resp = 550;
-  *flag = TRUE;
+  *flagp= TRUE;
   return US"XCLIENT command used when not advertised";
   }
 
 if (sender_address)
   {
   *resp = 503;
-  *flag = FALSE;
+  *flagp= FALSE;
   return US"mail transaction in progress";
   }
 
@@ -241,6 +243,7 @@ for (state = XCLIENT_SKIP_SPACES; *s; )
 	    { store_pool = old_pool; s = US"bad proto for XCLIENT"; goto fatal_501; }
 	  break;
 # endif
+	default: break;	/* stupid compiler silencing */
 	}
       store_pool = old_pool;
       state = XCLIENT_SKIP_SPACES;
@@ -270,7 +273,7 @@ proxy_session = TRUE;
 return NULL;
 
 fatal_501:
-  *flag = TRUE;
+  *flagp= TRUE;
   *resp = 501;
   return s;
 }
